import unittest
import mock
from maccli.helper.exception import MacParameterNotFound

from tests.mock_data import *
import maccli.helper.macfile


class HelperMacfileTestCase(unittest.TestCase):

    def test_has_dependencies_resources(self):
        command = "aws elb register-instances-with-load-balancer " \
               "--load-balancer-name my-load-balancer " \
               "--instances role.app.get_id infrastructure.app_inf.get_id"

        actual_dependencies = maccli.helper.macfile.has_dependencies(command, MOCK_PARSE_MACFILE_V2_EXPECTED_ROLES,
                                               MOCK_PARSE_MACFILE_V2_EXPECTED_INFRASTRUCTURES, MOCK_PARSE_MACFILE_V2_EXPECTED_ACTIONS)
        self.assertTrue(actual_dependencies)

    def test_has_dependencies_resources_2(self):
        command = "aws autoscaling create-launch-configuration --launch-configuration-name my-lc --image-id resource.create_image_inf.json.ImageId"

        actual_dependencies = maccli.helper.macfile.has_dependencies(command, MOCK_PARSE_MACFILE_V2_EXPECTED_ROLES,
                                                                     MOCK_PARSE_MACFILE_V2_EXPECTED_INFRASTRUCTURES, MOCK_PARSE_MACFILE_V2_EXPECTED_ACTIONS)
        self.assertTrue(actual_dependencies)

    def test_has_dependencies_infrastructure_params(self):
        command="aws elb create-load-balancer --load-balancer-name infrastructure.param.load-balancer-name --listeners infrastructure.listeners --security-groups sg-xxx --region us-east-1 --subnets subnet-yyy"
        INFRASTRUCTURES = OrderedDict([('build_lb_inf', OrderedDict([('resource', 'build_lb'), ('params', OrderedDict([('load-balancer-name', 'lb-local-parameters'), ('listerners', 'Protocol=HTTP,LoadBalancerPort=80,InstanceProtocol=HTTP,InstancePort=80')]))]))])
        actual_dependencies = maccli.helper.macfile.has_dependencies(command, [], INFRASTRUCTURES, [])
        self.assertTrue(actual_dependencies)

    def test_has_dependencies_infrastructures(self):
            #logging.basicConfig(level=logging.DEBUG)
            command = "aws elb create-load-balancer --load-balancer-name my-load-balancer --listeners Protocol=HTTP,LoadBalancerPort=80,InstanceProtocol=HTTP,InstancePort=80 --region infrastructure.app_inf.location --availability-zones role.app.get_id.get_availability_zone"
            actual_dependencies = maccli.helper.macfile.has_dependencies(command, MOCK_PARSE_MACFILE_V2_EXPECTED_ROLES,
                                                                         MOCK_PARSE_MACFILE_V2_EXPECTED_INFRASTRUCTURES, MOCK_PARSE_MACFILE_V2_EXPECTED_ACTIONS)
            self.assertTrue(actual_dependencies)

    def test_has_dependencies_action(self):
        ROLES=[]
        INFRASTRUCTURES=OrderedDict([('vpc_inf', OrderedDict([('resource', 'create_vpc')])), ('subnet_inf', OrderedDict([('resource', 'create_subnet')])), ('internet_gateway_inf', OrderedDict([('resource', 'create_internet_gateway')])), ('attach_route_to_internet_gateway', OrderedDict([('resource', 'create_route')]))])
        ACTIONS=OrderedDict([('get_id', OrderedDict([('bash', 'aws ec2 describe-route-tables --filters "Name=vpc-id,Values=resource.vpc_inf.json.Vpc.VpcId" --region us-east-1')]))])
        command = "aws ec2 create-route --route-table-id action.json.RouteTableId --destination-cidr-block 0.0.0.0/0"
        actual_dependencies = maccli.helper.macfile.has_dependencies(command, ROLES, INFRASTRUCTURES, ACTIONS)
        self.assertTrue(actual_dependencies)

    @mock.patch('maccli.service.instance.ssh_command_instance')
    def test_parse_env_resource(self, mock_run):
        CMD_RAW = 'aws elb register-instances-with-load-balancer --load-balancer-name my-load-balancer --instances role.app.get_id'
        CMD_CLEAN = 'aws elb register-instances-with-load-balancer --load-balancer-name my-load-balancer --instances i-3000b0e3'
        INSTANCES = [{u'status': u'Ready', u'servername': u'4oj-app-3000b0e3', u'lifespan': 3, u'ipv4': u'54.175.105.21', u'type': u'testing', u'id': u'44ojeae228l9d3aemcf4q2rcgk', u'metadata': {u'infrastructure': {u'macfile_infrastructure_name': u'app_inf', u'environment_raw': [{u'DB_IP': u'127.0.0.1'}, {u'APP_BRANCH': u'master'}], u'version': u'1.0', u'name': u'demo', u'macfile_role_name': u'app'}, u'system': {u'infrastructure': {u'hardware': u't1.micro', u'deployment': u'testing', u'location': u'us-east-1', u'lifespan': 60, u'provider': u'amazon'}, u'role': {u'environment': {u'APP_BRANCH': u'master', u'DB_IP': u'127.0.0.1'}, u'cookbook_tag': u'demo_application', u'block_tags': [{}]}}}}]
        mock_run.return_value = 0, "i-3000b0e3", None
        actual, processed = maccli.helper.macfile.parse_envs(CMD_RAW, INSTANCES, MOCK_PARSE_MACFILE_V2_EXPECTED_ROLES, MOCK_PARSE_MACFILE_V2_EXPECTED_INFRASTRUCTURES, MOCK_PARSE_MACFILE_V2_EXPECTED_ACTIONS, [])
        self.assertEqual(actual, CMD_CLEAN)
        self.assertTrue(processed)

    @mock.patch('maccli.service.instance.ssh_command_instance')
    def test_parse_env_resource_no_instances(self, mock_run):
        CMD_RAW = 'aws elb register-instances-with-load-balancer --load-balancer-name my-load-balancer --instances role.app.get_id'
        INSTANCES = []
        mock_run.return_value = 0, "", None
        actual, processed = maccli.helper.macfile.parse_envs(CMD_RAW, INSTANCES, MOCK_PARSE_MACFILE_V2_EXPECTED_ROLES, MOCK_PARSE_MACFILE_V2_EXPECTED_INFRASTRUCTURES, MOCK_PARSE_MACFILE_V2_EXPECTED_ACTIONS, [])
        self.assertFalse(processed)

    @mock.patch('maccli.service.instance.ssh_command_instance')
    def test_parse_env_infrastructure(self, mock_run):
        CMD_RAW = 'aws elb create-load-balancer --load-balancer-name my-load-balancer --listeners Protocol=HTTP,LoadBalancerPort=80,InstanceProtocol=HTTP,InstancePort=80 --region infrastructure.app_inf.location --availability-zones role.app.get_availability_zone'
        CMD_CLEAN = 'aws elb create-load-balancer --load-balancer-name my-load-balancer --listeners Protocol=HTTP,LoadBalancerPort=80,InstanceProtocol=HTTP,InstancePort=80 --region us-east-1 --availability-zones us-east-1e'
        INSTANCES = [{u'status': u'Ready', u'servername': u'4oj-app-3000b0e3', u'lifespan': 3, u'ipv4': u'54.175.105.21', u'type': u'testing', u'id': u'44ojeae228l9d3aemcf4q2rcgk', u'metadata': {u'infrastructure': {u'macfile_infrastructure_name': u'app_inf', u'environment_raw': [{u'DB_IP': u'127.0.0.1'}, {u'APP_BRANCH': u'master'}], u'version': u'1.0', u'name': u'demo', u'macfile_role_name': u'app'}, u'system': {u'infrastructure': {u'hardware': u't1.micro', u'deployment': u'testing', u'location': u'us-east-1', u'lifespan': 60, u'provider': u'amazon'}, u'role': {u'environment': {u'APP_BRANCH': u'master', u'DB_IP': u'127.0.0.1'}, u'cookbook_tag': u'demo_application', u'block_tags': [{}]}}}}]
        mock_run.return_value = 0, "us-east-1e", None
        actual, processed = maccli.helper.macfile.parse_envs(CMD_RAW, INSTANCES, MOCK_PARSE_MACFILE_V2_EXPECTED_ROLES, MOCK_PARSE_MACFILE_V2_EXPECTED_INFRASTRUCTURES, MOCK_PARSE_MACFILE_V2_1_EXPECTED_ACTIONS, [])
        self.assertEqual(actual, CMD_CLEAN)
        self.assertTrue(processed)

    @mock.patch('maccli.service.instance.ssh_command_instance')
    def test_parse_env_infrastructure_resource_json(self, mock_run):
        CMD_RAW = "aws autoscaling create-launch-configuration --launch-configuration-name my-lc --image-id resource.create_image_inf.json.ImageId --instance-type m1.small"
        INSTANCES = [{u'status': u'Ready', u'servername': u's51-app-b16f8c19', u'lifespan': 33, u'ipv4': u'52.6.97.27', u'type': u'testing', u'id': u'ps511bs6t72qfr20r4toahq7i8', u'metadata': {u'infrastructure': {u'macfile_infrastructure_name': u'image_base_inf', u'environment_raw': [{u'DB_IP': u'127.0.0.1'}, {u'APP_BRANCH': u'master'}], u'version': u'1.0', u'name': u'demo', u'macfile_role_name': u'app'}, u'system': {u'infrastructure': {u'hardware': u't1.micro', u'deployment': u'testing', u'location': u'us-east-1', u'lifespan': 60, u'provider': u'amazon'}, u'role': {u'environment': {u'APP_BRANCH': u'master', u'DB_IP': u'127.0.0.1'}, u'cookbook_tag': u'demo_application', u'block_tags': [{}]}}}}]
        ROLES = OrderedDict([('app', OrderedDict([('instance create', OrderedDict([('configuration', 'demo_application'), ('environment', [OrderedDict([('DB_IP', '127.0.0.1')]), OrderedDict([('APP_BRANCH', 'master')])])]))]))])
        INFRASTRUCTURES = OrderedDict([('image_base_inf', OrderedDict([('name', 'app'), ('provider', 'amazon'), ('location', 'us-east-1'), ('hardware', 't1.micro'), ('role', 'app'), ('amount', 1)])), ('create_image_inf', OrderedDict([('resource', 'create_image')])), ('create_launch_configuration_inf', OrderedDict([('resource', 'create_launch_configuration')]))])
        ACTIONS = OrderedDict([('get_id', OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/instance-id')]))])
        PROCESSED_RESOURCES = [{'create_image_inf': {'rc': 0, 'stderr': '', 'stdout': '{\n    "ImageId": "ami-4d2dee26"\n}\n'}}]
        CMD_CLEAN = 'aws autoscaling create-launch-configuration --launch-configuration-name my-lc --image-id ami-4d2dee26 --instance-type m1.small'
        mock_run.return_value = 0, "i-b16f8c19", None
        actual, processed = maccli.helper.macfile.parse_envs(CMD_RAW, INSTANCES, ROLES, INFRASTRUCTURES, ACTIONS, PROCESSED_RESOURCES)
        self.assertEqual(actual, CMD_CLEAN)
        self.assertTrue(processed)

    def test_parse_env_multilevel_json(self):
        CMD_RAW = "aws ec2 create-subnet --vpc-id resource.vpc_inf.json.Vpc.VpcId --cidr-block 10.0.1.0/24 --region us-east-1"
        INSTANCES = []
        ROLES = []
        INFRASTRUCTURES = OrderedDict([('vpc_inf', OrderedDict([('resource', 'create_vpc')])), ('subnet_inf', OrderedDict([('resource', 'create_subnet')])), ('internet_gateway_inf', OrderedDict([('resource', 'create_internet_gateway')])), ('attach_internet_gateway_inf', OrderedDict([('resource', 'create_internet_gateway')]))])
        ACTIONS = OrderedDict([('get_id', OrderedDict([('bash', 'aws ec2 describe-route-tables --filters "Name=vpc-id,Values=resource.vpc_inf.json.Vpc.VpcId" --region us-east-1')]))])
        PROCESSED_RESOURCES = [{'vpc_inf': {'rc': 0, 'stderr': '', 'stdout': '{\n    "Vpc": {\n        "InstanceTenancy": "default", \n        "State": "pending", \n        "VpcId": "vpc-7257a716", \n        "CidrBlock": "10.0.0.0/16", \n        "DhcpOptionsId": "dopt-838273e6"\n    }\n}\n'}}]
        CMD_CLEAN = 'aws ec2 create-subnet --vpc-id vpc-7257a716 --cidr-block 10.0.1.0/24 --region us-east-1'
        actual, processed = maccli.helper.macfile.parse_envs(CMD_RAW, INSTANCES, ROLES, INFRASTRUCTURES, ACTIONS, PROCESSED_RESOURCES)
        self.assertEqual(actual, CMD_CLEAN)
        self.assertTrue(processed)

    @mock.patch('maccli.helper.cmd.run')
    def test_parse_action_json(self, mock_run):
        CMD_RAW = "aws ec2 create-route --route-table-id action.get_route.json.RouteTables.0.RouteTableId --destination-cidr-block 0.0.0.0/0 --gateway-id resource.internet_gateway_inf.json.InternetGateway.InternetGatewayId --region us-east-1"
        INSTANCES = []
        ROLES = []
        INFRASTRUCTURES = OrderedDict([('vpc_inf', OrderedDict([('resource', 'create_vpc')])), ('subnet_inf', OrderedDict([('resource', 'create_subnet')])), ('internet_gateway_inf', OrderedDict([('resource', 'create_internet_gateway')])), ('attach_route_to_internet_gateway', OrderedDict([('resource', 'create_route')]))])
        ACTIONS = OrderedDict([('get_route', OrderedDict([('bash', 'aws ec2 describe-route-tables --filters "Name=vpc-id,Values=resource.vpc_inf.json.Vpc.VpcId" --region us-east-1')]))])
        PROCESSED_RESOURCES = [{'vpc_inf': {'cmd': 'aws ec2 create-vpc --cidr-block 10.0.0.0/16 --region us-east-1', 'rc': 0, 'stderr': '', 'stdout': '{\n    "Vpc": {\n        "InstanceTenancy": "default", \n        "State": "pending", \n        "VpcId": "vpc-ee45b58a", \n        "CidrBlock": "10.0.0.0/16", \n        "DhcpOptionsId": "dopt-838273e6"\n    }\n}\n'}}, {'subnet_inf': {'cmd': u'aws ec2 create-subnet --vpc-id vpc-ee45b58a --cidr-block 10.0.1.0/24 --region us-east-1', 'rc': 0, 'stderr': '', 'stdout': '{\n    "Subnet": {\n        "VpcId": "vpc-ee45b58a", \n        "CidrBlock": "10.0.1.0/24", \n        "State": "pending", \n        "AvailabilityZone": "us-east-1c", \n        "SubnetId": "subnet-97fcf9e0", \n        "AvailableIpAddressCount": 251\n    }\n}\n'}}, {'internet_gateway_inf': {'cmd': 'aws ec2 create-internet-gateway --region us-east-1', 'rc': 0, 'stderr': '', 'stdout': '{\n    "InternetGateway": {\n        "Tags": [], \n        "InternetGatewayId": "igw-4b99bf2e", \n        "Attachments": []\n    }\n}\n'}}]
        CMD_CLEAN = 'aws ec2 create-route --route-table-id rtb-147bde70 --destination-cidr-block 0.0.0.0/0 --gateway-id igw-4b99bf2e --region us-east-1'
        mock_run.return_value = 0, AWS_DESCRIBE_ROUTE_RAW, None
        actual, processed = maccli.helper.macfile.parse_envs(CMD_RAW, INSTANCES, ROLES, INFRASTRUCTURES, ACTIONS, PROCESSED_RESOURCES)
        self.assertEqual(CMD_CLEAN, actual)
        self.assertTrue(processed)

    def test_parse_parameter_not_found(self):
        CMD_RAW = "aws rds create-db-subnet-group --db-subnet-group-name mac-dev --db-subnet-group-description mac-dev --subnet-ids resource.subnet_2_inf.json.Subnet.SubnetId resource.subnet_inf.json.Subnet.SubnetId --region us-east-1"
        INSTANCES = []
        ROLES = []
        INFRASTRUCTURES = OrderedDict([('vpc_inf', OrderedDict([('resource', 'create_vpc')])), ('vpc_enable_hostname_dns_inf', OrderedDict([('resource', 'enable_hostname_resolution')])), ('subnet_inf', OrderedDict([('resource', 'create_subnet')])), ('subnet2_inf', OrderedDict([('resource', 'create_subnet_2')])), ('subnet_enable_public_ips_inf', OrderedDict([('resource', 'subnet_enable_public_ips')])), ('subnet_enable_public_ips_2_inf', OrderedDict([('resource', 'subnet_enable_public_ips_2')])), ('internet_gateway_inf', OrderedDict([('resource', 'create_internet_gateway')])), ('attach_internet_gateway_to_vpc', OrderedDict([('resource', 'attach_internet_gateway')])), ('attach_route_to_internet_gateway', OrderedDict([('resource', 'create_route')])), ('authorize_ssh_inf', OrderedDict([('resource', 'authorize_ssh')])), ('create_rds_subnet_group_inf', OrderedDict([('resource', 'create_rds_subnet_group')])), ('create_rds_inf', OrderedDict([('resource', 'create_rds')]))])
        ACTIONS = OrderedDict([('get_route', OrderedDict([('bash', 'aws ec2 describe-route-tables --filters Name=vpc-id,Values=resource.vpc_inf.json.Vpc.VpcId --region us-east-1')])), ('get_security_group_id', OrderedDict([('bash', 'aws ec2 describe-security-groups --filters Name=vpc-id,Values=resource.vpc_inf.json.Vpc.VpcId --region us-east-1')]))])
        PROCESSED_RESOURCES = [{'vpc_inf': {'cmd': 'aws ec2 create-vpc --cidr-block 10.0.0.0/16 --region us-east-1', 'rc': 0, 'stderr': '', 'stdout': '{\n    "Vpc": {\n        "InstanceTenancy": "default", \n        "State": "pending", \n        "VpcId": "vpc-82e311e6", \n        "CidrBlock": "10.0.0.0/16", \n        "DhcpOptionsId": "dopt-838273e6"\n    }\n}\n'}}, {'vpc_enable_hostname_dns_inf': {'cmd': u'aws ec2 modify-vpc-attribute --vpc-id vpc-82e311e6 --enable-dns-hostnames --region us-east-1', 'rc': 0, 'stderr': '', 'stdout': ''}}, {'subnet_inf': {'cmd': u'aws ec2 create-subnet --vpc-id vpc-82e311e6 --cidr-block 10.0.1.0/24 --region us-east-1', 'rc': 0, 'stderr': '', 'stdout': '{\n    "Subnet": {\n        "VpcId": "vpc-82e311e6", \n        "CidrBlock": "10.0.1.0/24", \n        "State": "pending", \n        "AvailabilityZone": "us-east-1b", \n        "SubnetId": "subnet-aa327081", \n        "AvailableIpAddressCount": 251\n    }\n}\n'}}, {'subnet2_inf': {'cmd': u'aws ec2 create-subnet --vpc-id vpc-82e311e6 --cidr-block 10.0.2.0/24 --availability-zone us-east-1b --region us-east-1', 'rc': 0, 'stderr': '', 'stdout': '{\n    "Subnet": {\n        "VpcId": "vpc-82e311e6", \n        "CidrBlock": "10.0.2.0/24", \n        "State": "pending", \n        "AvailabilityZone": "us-east-1b", \n        "SubnetId": "subnet-a8327083", \n        "AvailableIpAddressCount": 251\n    }\n}\n'}}, {'subnet_enable_public_ips_inf': {'cmd': u'aws ec2 modify-subnet-attribute --subnet-id subnet-aa327081 --map-public-ip-on-launch --region us-east-1', 'rc': 0, 'stderr': '', 'stdout': ''}}, {'subnet_enable_public_ips_2_inf': {'cmd': u'aws ec2 modify-subnet-attribute --subnet-id subnet-a8327083 --map-public-ip-on-launch --region us-east-1', 'rc': 0, 'stderr': '', 'stdout': ''}}, {'internet_gateway_inf': {'cmd': 'aws ec2 create-internet-gateway --region us-east-1', 'rc': 0, 'stderr': '', 'stdout': '{\n    "InternetGateway": {\n        "Tags": [], \n        "InternetGatewayId": "igw-d86141bd", \n        "Attachments": []\n    }\n}\n'}}, {'attach_internet_gateway_to_vpc': {'cmd': u'aws ec2 attach-internet-gateway --internet-gateway-id igw-d86141bd --vpc-id vpc-82e311e6 --region us-east-1', 'rc': 0, 'stderr': '', 'stdout': ''}}, {'attach_route_to_internet_gateway': {'cmd': u'aws ec2 create-route --route-table-id rtb-26d27542 --destination-cidr-block 0.0.0.0/0 --gateway-id igw-d86141bd --region us-east-1', 'rc': 0, 'stderr': '', 'stdout': '{\n    "Return": true\n}\n'}}, {'authorize_ssh_inf': {'cmd': u'aws ec2 authorize-security-group-ingress --group-id sg-e54b3d82 --protocol tcp --port 22 --cidr 0.0.0.0/0 --region us-east-1', 'rc': 0, 'stderr': '', 'stdout': ''}}]
        self.assertRaises(MacParameterNotFound, maccli.helper.macfile.parse_envs, CMD_RAW, INSTANCES, ROLES, INFRASTRUCTURES, ACTIONS, PROCESSED_RESOURCES)

    def test_parse_envs_destroy(self):
        CMD_CLEAN = "aws ec2 delete-vpc --vpc-id vpc-b78c48d3"
        INSTANCES = []
        RESOURCES = [{u'create': {u'cmd': u'aws ec2 create-vpc --cidr-block 10.0.0.0/16 --region us-east-1', u'rc': 0, u'stderr': u'', u'stdout': u'{\n    "Vpc": {\n        "InstanceTenancy": "default", \n        "State": "pending", \n        "VpcId": "vpc-b78c48d3", \n        "CidrBlock": "10.0.0.0/16", \n        "DhcpOptionsId": "dopt-838273e6"\n    }\n}\n'}, u'metadata': {u'infrastructure': {u'macfile_infrastructure_name': u'vpc_inf', u'version': u'1.0', u'name': u'demo', u'macfile_resource_name': u'create_vpc'}}, u'cmdDestroy': u'aws ec2 delete-vpc --vpc-id resource.vpc_inf.json.Vpc.VpcId', u'name': u'vpc_inf', u'cmdCreate': u'aws ec2 create-vpc --cidr-block 10.0.0.0/16 --region us-east-1'}]
        RESOURCE = {u'create': {u'cmd': u'aws ec2 create-vpc --cidr-block 10.0.0.0/16 --region us-east-1', u'rc': 0, u'stderr': u'', u'stdout': u'{\n    "Vpc": {\n        "InstanceTenancy": "default", \n        "State": "pending", \n        "VpcId": "vpc-b78c48d3", \n        "CidrBlock": "10.0.0.0/16", \n        "DhcpOptionsId": "dopt-838273e6"\n    }\n}\n'}, u'metadata': {u'infrastructure': {u'macfile_infrastructure_name': u'vpc_inf', u'version': u'1.0', u'name': u'demo', u'macfile_resource_name': u'create_vpc'}}, u'cmdDestroy': u'aws ec2 delete-vpc --vpc-id resource.vpc_inf.json.Vpc.VpcId', u'name': u'vpc_inf', u'cmdCreate': u'aws ec2 create-vpc --cidr-block 10.0.0.0/16 --region us-east-1'}
        actual = maccli.helper.macfile.parse_envs_destroy(RESOURCE, INSTANCES, RESOURCES)
        self.assertEqual(CMD_CLEAN, actual)

    def test_parse_envs_destroy_text(self):
        CMD_CLEAN = "azure network vnet delete -q testnet --json"
        RESOURCE = {u'create': {u'cmd': u'azure network vnet create infrastructure.param.name --location "infrastructure.param.location" testnet --json && echo "infrastructure.param.name"', u'rc': 0, u'stderr': u'', u'stdout': u'testnet\n'}, u'metadata': {u'infrastructure': {u'macfile_infrastructure_name': u'create_network', u'macfile_infrastructure_params': {u'name': u'testnet', u'location': u'East US'}, u'version': u'1.0', u'name': u'azure_demo', u'macfile_resource_name': u'network'}}, u'cmdDestroy': u'azure network vnet delete -q resource.create_network.text.regex(.*) --json', u'name': u'create_network', u'cmdCreate': u'azure network vnet create infrastructure.param.name --location "infrastructure.param.location" testnet --json && echo "infrastructure.param.name"'}
        INSTANCES = []
        RESOURCES = [{u'create': {u'cmd': u'azure network vnet create infrastructure.param.name --location "infrastructure.param.location" testnet --json && echo "infrastructure.param.name"', u'rc': 0, u'stderr': u'', u'stdout': u'testnet\n'}, u'metadata': {u'infrastructure': {u'macfile_infrastructure_name': u'create_network', u'macfile_infrastructure_params': {u'name': u'testnet', u'location': u'East US'}, u'version': u'1.0', u'name': u'azure_demo', u'macfile_resource_name': u'network'}}, u'cmdDestroy': u'azure network vnet delete -q resource.create_network.text.regex(.*) --json', u'name': u'create_network', u'cmdCreate': u'azure network vnet create infrastructure.param.name --location "infrastructure.param.location" testnet --json && echo "infrastructure.param.name"'}]
        actual = maccli.helper.macfile.parse_envs_destroy(RESOURCE, INSTANCES, RESOURCES)
        self.assertEqual(CMD_CLEAN, actual)


    def test_parse_envs_infrastructure_params_destroy(self):
        CMD_CLEAN = "aws elb delete-load-balancer --load-balancer-name lb-local-parameters --region us-east-1"
        INSTANCES = []
        RESOURCES = [{u'create': {u'cmd': u'aws elb create-load-balancer --load-balancer-name infrastructure.param.load-balancer-name --listeners infrastructure.param.listeners --availability-zones us-east-1b --region us-east-1', u'rc': 0, u'stderr': u'', u'stdout': u'{\n    "DNSName": "lb-local-parameters-1322457308.us-east-1.elb.amazonaws.com"\n}\n'}, u'metadata': {u'infrastructure': {u'macfile_infrastructure_name': u'build_lb_inf', u'macfile_infrastructure_params': {u'listeners': u'Protocol=HTTP,LoadBalancerPort=80,InstanceProtocol=HTTP,InstancePort=80', u'load-balancer-name': u'lb-local-parameters'}, u'version': u'test', u'name': u'test', u'macfile_resource_name': u'build_lb'}}, u'cmdDestroy': u'aws elb delete-load-balancer --load-balancer-name infrastructure.param.load-balancer-name --region us-east-1', u'name': u'build_lb_inf', u'cmdCreate': u'aws elb create-load-balancer --load-balancer-name infrastructure.param.load-balancer-name --listeners infrastructure.param.listeners --availability-zones us-east-1b --region us-east-1'}]
        RESOURCE_TO_DESTROY = {u'create': {u'cmd': u'aws elb create-load-balancer --load-balancer-name infrastructure.param.load-balancer-name --listeners infrastructure.param.listeners --availability-zones us-east-1b --region us-east-1', u'rc': 0, u'stderr': u'', u'stdout': u'{\n    "DNSName": "lb-local-parameters-1322457308.us-east-1.elb.amazonaws.com"\n}\n'}, u'metadata': {u'infrastructure': {u'macfile_infrastructure_name': u'build_lb_inf', u'macfile_infrastructure_params': {u'listeners': u'Protocol=HTTP,LoadBalancerPort=80,InstanceProtocol=HTTP,InstancePort=80', u'load-balancer-name': u'lb-local-parameters'}, u'version': u'test', u'name': u'test', u'macfile_resource_name': u'build_lb'}}, u'cmdDestroy': u'aws elb delete-load-balancer --load-balancer-name infrastructure.param.load-balancer-name --region us-east-1', u'name': u'build_lb_inf', u'cmdCreate': u'aws elb create-load-balancer --load-balancer-name infrastructure.param.load-balancer-name --listeners infrastructure.param.listeners --availability-zones us-east-1b --region us-east-1'}
        actual = maccli.helper.macfile.parse_envs_destroy(RESOURCE_TO_DESTROY, INSTANCES, RESOURCES)
        self.assertEqual(CMD_CLEAN, actual)


    @mock.patch('maccli.helper.cmd.run')
    def test_parse_action_regex(self, mock_run):
        CMD_RAW = "command with parameter resource.bucket_inf.text.regex(s3://(.*)?/)"
        INSTANCES = []
        ROLES = []
        INFRASTRUCTURES = OrderedDict([('bucket_inf', OrderedDict([('resource', 'bucket')])), ('this_fails_inf', OrderedDict([('resource', 'this_fails')]))])
        ACTIONS = []
        PROCESSED_RESOURCES = [{'bucket_inf': {'cmd': 'aws s3 mb s3://s3-dev10 --region us-east-1', 'rc': 0, 'stderr': '', 'stdout': 'make_bucket: s3://s3-dev10/\n'}}]
        CMD_CLEAN = 'command with parameter s3-dev10'
        mock_run.return_value = 0, AWS_DESCRIBE_ROUTE_RAW, None
        actual, processed = maccli.helper.macfile.parse_envs(CMD_RAW, INSTANCES, ROLES, INFRASTRUCTURES, ACTIONS, PROCESSED_RESOURCES)
        self.assertEqual(CMD_CLEAN, actual)
        self.assertTrue(processed)

    @mock.patch('maccli.helper.cmd.run')
    def test_parse_resource_params(self, mock_run):
        CMD_RAW = "aws elb create-load-balancer --load-balancer-name infrastructure.param.load-balancer-name --listeners infrastructure.param.listeners --security-groups sg-xxx --region us-east-1 --subnets subnet-yyy"
        INSTANCES = []
        ROLES = []
        INFRASTRUCTURES = OrderedDict([('build_lb_inf', OrderedDict([('resource', 'build_lb'), ('params', OrderedDict([('load-balancer-name', 'lb-local-parameters'), ('listeners', 'Protocol=HTTP,LoadBalancerPort=80,InstanceProtocol=HTTP,InstancePort=80')]))]))])
        ACTIONS = []
        PROCESSED_RESOURCES = []
        INFRASTRUCTURE = INFRASTRUCTURES['build_lb_inf']
        CMD_CLEAN = 'aws elb create-load-balancer --load-balancer-name lb-local-parameters --listeners Protocol=HTTP,LoadBalancerPort=80,InstanceProtocol=HTTP,InstancePort=80 --security-groups sg-xxx --region us-east-1 --subnets subnet-yyy'
        mock_run.return_value = 0, AWS_DESCRIBE_ROUTE_RAW, None
        actual, processed = maccli.helper.macfile.parse_envs(CMD_RAW, INSTANCES, ROLES, INFRASTRUCTURES, ACTIONS, PROCESSED_RESOURCES, INFRASTRUCTURE)
        self.assertEqual(CMD_CLEAN, actual)
        self.assertTrue(processed)

    @mock.patch('maccli.helper.cmd.run')
    def test_parse_action_regex(self, mock_run):
        CMD_RAW = "ssh ubuntu@action.get_public_ip.text.regex(\"PublicIp\":\"([0-9\.]+)\") -i /tmp/privkey 'sudo apt-get update && sudo apt-get install apache2 -y'"
        INSTANCES = []
        ROLES = []
        INFRASTRUCTURES = OrderedDict([('bootstrap_inf', OrderedDict([('action', 'bootstrap')]))])
        ACTIONS = OrderedDict([('set_private_key', OrderedDict([('bash', 'cat >/tmp/privKey <<EOL\nresource.ssh_pair.json.KeyMaterial\nEOL\nchmod 600 /tmp/privKey\n')])), ('bootstrap', OrderedDict([('bash', 'ssh ubuntu@action.get_public_ip.text.regex("PublicIp": "([0-9\\.]+)") -i /tmp/privkey \'sudo apt-get update && sudo apt-get install apache2 -y\'')])), ('get_public_ip', OrderedDict([('bash', 'aws ec2 describe-instances --instance-ids i-1c33b6a3')]))])
        PROCESSED_RESOURCES = []
        INFRASTRUCTURE = OrderedDict([('action', 'bootstrap')])
        CMD_CLEAN = "ssh ubuntu@52.91.191.63 -i /tmp/privkey 'sudo apt-get update && sudo apt-get install apache2 -y'"
        mock_run.return_value = 0, AWS_DESCRIBE_EC2_INSTANCE, None
        actual, processed = maccli.helper.macfile.parse_envs(CMD_RAW, INSTANCES, ROLES, INFRASTRUCTURES, ACTIONS, PROCESSED_RESOURCES, INFRASTRUCTURE)
        self.assertEqual(CMD_CLEAN, actual)
        self.assertTrue(processed)

    @mock.patch('maccli.helper.cmd.run')
    def test_parse_param_does_not_esists(self, mock_run):
        CMD_RAW = "azure vm create --vm-name infrastructure.param.name --location infrastructure.param.location --virtual-network-name infrastructure.param.virtual-network-name --rdp 3389 infrastructure.param.name infrastructure.param.image infrastructure.param.username infrastructure.param.password"
        INSTANCES = []
        ROLES = []
        INFRASTRUCTURES = OrderedDict([('create_windows_vm', OrderedDict([('resource', 'windows_vm'), ('params', OrderedDict([('vm-name', 'demo_test'), ('location', 'East US'), ('virtual-network-name', 'testnet'), ('image', 'ad072bd3082149369c449ba5832401ae__Windows-Server-Remote-Desktop-Session-Host-on-Windows-Server-2012-R2-20150828-0350'), ('username', 'username'), ('password', 'MySafePassword!')]))])), ('provision_windows_vm', OrderedDict([('action', 'provision_windows_1')]))])
        ACTIONS = OrderedDict([('provision_windows_1', OrderedDict([('bash', '"azure vm extension set demo_test CustomScriptExtension Microsoft.Compute 1.4 -i \'{"fileUris":["https://gist.githubusercontent.com/tk421/11a2ec3fad5bcce5de3f/raw/b0ca8770100905a39b2dadd0fec7d159ccc73874/createFolder.ps1"], "commandToExecute": "powershell -ExecutionPolicy Unrestricted -file createFolder.ps1" }\' --json"\n')]))])
        PROCESSED_RESOURCES = []
        INFRASTRUCTURE = OrderedDict([('resource', 'windows_vm'), ('params', OrderedDict([('vm-name', 'demo_test'), ('location', 'East US'), ('virtual-network-name', 'testnet'), ('image', 'ad072bd3082149369c449ba5832401ae__Windows-Server-Remote-Desktop-Session-Host-on-Windows-Server-2012-R2-20150828-0350'), ('username', 'username'), ('password', 'MySafePassword!')]))])
        mock_run.return_value = 0, AWS_DESCRIBE_EC2_INSTANCE, None
        self.assertRaises(MacParameterNotFound, maccli.helper.macfile.parse_envs, CMD_RAW, INSTANCES, ROLES, INFRASTRUCTURES, ACTIONS, PROCESSED_RESOURCES, INFRASTRUCTURE)

    @mock.patch('maccli.helper.cmd.run')
    def test_parse_action_text(self, mock_run):
        CMD_RAW = 'aws ec2 describe-subnets --output text | grep action.get_first_vpc.text.regex(.*) | cut -f8 | tr \'\n\' \' \''
        INSTANCES = []
        ROLES = []
        INFRASTRUCTURES = OrderedDict([('create_rds_subnet', OrderedDict([('resource', 'rds_subnet_group'), ('params', OrderedDict([('name', 'my_rds_subnet')]))])), ('create rds', OrderedDict([('resource', 'rds'), ('params', OrderedDict([('name', 'rds-demo'), ('master-username', 'myuser'), ('master-user-password', 'mypass')]))]))])
        ACTIONS = OrderedDict([('get_first_vpc', OrderedDict([('bash', 'aws ec2 describe-subnets --output text | head -n1 | cut -f9')])), ('get_subnets', OrderedDict([('bash', "aws ec2 describe-subnets --output text | grep action.get_first_vpc.text.regex(.*) | cut -f8 | tr '\\n' ' '")]))])
        PROCESSED_RESOURCES = []
        INFRASTRUCTURE = OrderedDict([('resource', 'rds_subnet_group'), ('params', OrderedDict([('name', 'my_rds_subnet')]))])
        CMD_CLEAN = 'aws ec2 describe-subnets --output text | grep vpc-ffc1ca9a | cut -f8 | tr \'\n\' \' \''
        mock_run.return_value = 0, "vpc-ffc1ca9a", None
        actual, processed = maccli.helper.macfile.parse_envs(CMD_RAW, INSTANCES, ROLES, INFRASTRUCTURES, ACTIONS, PROCESSED_RESOURCES, INFRASTRUCTURE)
        self.assertEqual(CMD_CLEAN, actual)
        self.assertTrue(processed)

    @mock.patch('maccli.helper.cmd.run')
    def test_param_integer(self, mock_run):
        CMD_RAW = "aws rds create-db-instance --db-instance-identifier infrastructure.param.name --allocated-storage infrastructure.param.allocated-storage --db-instance-class infrastructure.param.db-instance-class --engine infrastructure.param.engine --engine-version infrastructure.param.engine-version --master-username infrastructure.param.master-username --master-user-password infrastructure.param.master-user-password --db-subnet-group-name resource.create_rds_subnet.json.DBSubnetGroup.DBSubnetGroupName --auto-minor-version-upgrade --region us-east-1"
        INSTANCES = []
        ROLES = []
        INFRASTRUCTURES = OrderedDict([('create_rds_subnet', OrderedDict([('resource', 'rds_subnet_group'), ('params', OrderedDict([('name', 'my_rds_subnet')]))])), ('create_rds', OrderedDict([('resource', 'rds'), ('params', OrderedDict([('name', 'rds-demo'), ('master-username', 'myuser'), ('master-user-password', 'mysecretpass'), ('db-instance-class', 'db.t1.micro'), ('allocated-storage', 20), ('engine', 'mysql'), ('engine-version', '5.6.23')]))])), ('is_rd_built', OrderedDict([('action', 'wait_for_rds')]))])
        ACTIONS = OrderedDict([('get_first_vpc', OrderedDict([('bash', 'aws ec2 describe-subnets --output text | head -n1 | cut -f9')])), ('get_subnets', OrderedDict([('bash', "aws ec2 describe-subnets --output text | grep action.get_first_vpc.text.regex(.*) | cut -f8 | tr '\\n' ' '")])), ('wait_for_rds', OrderedDict([('bash', 'while ! aws rds describe-db-instances --output text | grep ENDPOINT; do sleep 10; done')]))])
        PROCESSED_RESOURCES = []
        INFRASTRUCTURE = OrderedDict([('resource', 'rds'), ('params', OrderedDict([('name', 'rds-demo'), ('master-username', 'myuser'), ('master-user-password', 'mysecretpass'), ('db-instance-class', 'db.t1.micro'), ('allocated-storage', 20), ('engine', 'mysql'), ('engine-version', '5.6.23')]))])
        CMD_CLEAN = "aws rds create-db-instance --db-instance-identifier rds-demo --allocated-storage 20 --db-instance-class db.t1.micro --engine mysql --engine-version mysql-version --master-username myuser --master-user-password mysecretpass --db-subnet-group-name resource.create_rds_subnet.json.DBSubnetGroup.DBSubnetGroupName --auto-minor-version-upgrade --region us-east-1"
        mock_run.return_value = 0, "MOCK_RETURN", None
        actual, processed = maccli.helper.macfile.parse_envs(CMD_RAW, INSTANCES, ROLES, INFRASTRUCTURES, ACTIONS, PROCESSED_RESOURCES, INFRASTRUCTURE)
        self.assertEqual(CMD_CLEAN, actual)
        self.assertFalse(processed)



from urllib.parse import urlencode

import maccli.helper.http


def get_user_configuration():
    status_code, json, raw = maccli.helper.http.send_request("GET", "/configuration/list")

    return status_code, json


def search_public_configuration(keywords):

    querystring = ""
    if keywords is not None and len(keywords):
        params = {
            'keyword': keywords
        }
        querystring = "?" + urlencode(params, True)

    status_code, json_response, raw = maccli.helper.http.send_request("GET", "/configuration/search%s" % querystring)

    return status_code, json_response




import unittest

import mock

import maccli.view.view_instance
from tests.mock_data import *


INSTANCES_RAW=[{u'status': u'Creating instance', u'servername': u'memcache', u'lifespan': 89, u'ipv4': u'', u'type': u'testing', u'id': u'nnrtg49kji92rgl56288rl06ml', u'metadata': {u'infrastructure': {u'macfile_infrastructure_name': u'shared_memcache', u'macfile_role_name': u'shared_memcache', u'version': u'1.0', u'name': u'testing'}, u'system': {u'infrastructure': {u'hardware': u'512mb', u'deployment': u'testing', u'location': u'sfo1', u'lifespan': 90, u'provider': u'manageacloud'}, u'role': {}}}}, {u'status': u'Creating instance', u'servername': u'engine', u'lifespan': 89, u'ipv4': u'', u'type': u'testing', u'id': u'm9jj37g32mqhb90ftmpqhl4mtc', u'metadata': {u'infrastructure': {u'macfile_infrastructure_name': u'engine', u'environment_raw': [{u'SHARED_MEMCACHE_IP': u'shared_memcache.PRIVATE_IP'}], u'version': u'1.0', u'name': u'testing', u'macfile_role_name': u'engine'}, u'system': {u'infrastructure': {u'hardware': u'512mb', u'deployment': u'testing', u'location': u'sfo1', u'lifespan': 90, u'provider': u'manageacloud'}, u'role': {u'environment': {u'SHARED_MEMCACHE_IP': u'shared_memcache.PRIVATE_IP'}}}}}]

class AuthTestCase(unittest.TestCase):

    def test_show_processing_instances(self):
        maccli.view.view_instance.show_processing_instances(INSTANCES_RAW)




import unittest

import mock
from argparse import ArgumentTypeError

from tests.mock_data import *
import maccli.parser_cli


class ParserCliTestCase(unittest.TestCase):

    def test_validate_environment(self):
        self.assertRaises(ArgumentTypeError, maccli.parser_cli.validate_environment, "invalid")
        self.assertRaises(ArgumentTypeError, maccli.parser_cli.validate_environment, "invalid = spaces")
        self.assertRaises(ArgumentTypeError, maccli.parser_cli.validate_environment, "BENCH_CREATION=")
        self.assertEqual(maccli.parser_cli.validate_environment("UNO=dos"), {'UNO':'dos'})
        self.assertEqual(maccli.parser_cli.validate_environment("uno=dos"), {'uno':'dos'})
        self.assertEqual(maccli.parser_cli.validate_environment("A_VALUE=dos2"), {'A_VALUE':'dos2'})
        self.assertEqual(maccli.parser_cli.validate_environment("a_value=dos2"), {'a_value':'dos2'})
        self.assertEqual(maccli.parser_cli.validate_environment("a_value=dos2=3"), {'a_value':'dos2=3'})
        self.assertEqual(maccli.parser_cli.validate_environment("""a_value=UNO
        DOS
        TRES"""), {'a_value':'''UNO
        DOS
        TRES'''})
        self.assertEqual(maccli.parser_cli.validate_environment("BENCH_CREATION=-i -s 70"), {'BENCH_CREATION':'-i -s 70'})

    def test_validate_hd(self):
        self.assertEqual(maccli.parser_cli.validate_hd("/dev/sda1:100"), {'/dev/sda1':'100'})
        self.assertEqual(maccli.parser_cli.validate_hd("/dev/sda1:50"), {'/dev/sda1':'50'})
        self.assertEqual(maccli.parser_cli.validate_hd("attachment:50:ssd"), {'attachment':'50:ssd'})
        self.assertEqual(maccli.parser_cli.validate_hd("/dev/ok:100"), {'/dev/ok':'100'})
        self.assertEqual(maccli.parser_cli.validate_hd("/dev/sda1:100:ok"), {'/dev/sda1':'100:ok'})
        self.assertEqual(maccli.parser_cli.validate_hd("/dev/sda1:100:ok:1000"), {'/dev/sda1':'100:ok:1000'})
        #self.assertRaises(ArgumentTypeError, maccli.parser_cli.validate_hd, "/dev/not/ok:100")
        #self.assertRaises(ArgumentTypeError, maccli.parser_cli.validate_hd, "/not/ok:100")
        self.assertRaises(ArgumentTypeError, maccli.parser_cli.validate_hd, "/dev/ok:wtf")
        self.assertRaises(ArgumentTypeError, maccli.parser_cli.validate_hd, "/dev/ok")
        self.assertRaises(ArgumentTypeError, maccli.parser_cli.validate_hd, "100")
        self.assertRaises(ArgumentTypeError, maccli.parser_cli.validate_hd, "/dev/sda1:100:not-ok")

    def test_validate_port(self):
        self.assertEqual(maccli.parser_cli.validate_port("22"), [22])
        self.assertEqual(maccli.parser_cli.validate_port("22,8080"), [22,8080])
        self.assertEqual(maccli.parser_cli.validate_port("1,22,8080,65535"), [1,22,8080,65535])
        self.assertRaises(ArgumentTypeError, maccli.parser_cli.validate_port, "0,22,8080,65535")
        self.assertRaises(ArgumentTypeError, maccli.parser_cli.validate_port, "22,8080,65536")
        self.assertRaises(ArgumentTypeError, maccli.parser_cli.validate_port, "22,sssss8080")
        self.assertRaises(ArgumentTypeError, maccli.parser_cli.validate_port, "sssss8080")



from prettytable import PrettyTable
import maccli


def show_infrastructure(infrastructure):
    pretty = PrettyTable(["Infrastructure name", "version"])
    if len(infrastructure):
        for inf in infrastructure:
            for version in inf['versions']:
                pretty.add_row([inf['name'], version])
        print(pretty)
    else:
        print("There is no active infrastructure")


def show_infrastructure_instances(infrastructure):
    pretty = PrettyTable(["Infrastructure name", "Version", "Instance name", "Instance id", "Status"])
    is_output = False
    for inf in infrastructure:
        for version in inf['versions']:
            for instance in inf['cloudServers']:
                instance_infrastructure_version = instance['metadata']['infrastructure']['version']
                if version == instance_infrastructure_version:
                    if instance['type'] == 'testing' and instance['status'] == "Ready":
                        status = "%s (%im left)" % (instance['status'], instance['lifespan'])
                    else:
                        status = instance['status']
                    pretty.add_row([inf['name'], version, instance['servername'], instance['id'], status])
                    is_output = True
    if is_output:
        print(pretty)
    else:
        print("There are no active instances in infrastructure")


def show_resources_in_infrastructure(infrastructures):

    """  Display resources in infrastrucrures """
    if len(infrastructures):
        pretty = PrettyTable(["Infrastructure name", "Version", "Resource type", "Resource name", "Status"])
        pretty.align = "l"

        is_output = False
        for inf in infrastructures:
            for version in inf['versions']:
                for resource in inf['resources']:
                    resource_type = resource['metadata']['infrastructure']['macfile_resource_name']
                    resource_name = resource['metadata']['infrastructure']['macfile_infrastructure_name']
                    resource_version = resource['metadata']['infrastructure']['version']
                    if version == resource_version:
                        status = ""
                        if 'destroy' in resource:
                            if resource['create']['rc']:
                                status = "Destroy failed"
                            else:
                                status = "Destroyed"

                        elif 'create' in resource:
                            if resource['create']['rc']:
                                status = "Creation failed"
                            else:
                                status = "Ready"

                        pretty.add_row([inf['name'], version, resource_type, resource_name, status])
                        is_output = True

        if is_output:
            print(pretty)
        else:
            print("There is no resources in infrastructure")


def show_infrastructure_resources(infrastructures, infrastructures_resources_processed):
    """ display table with infrastructure when it is being processed """

    if len(infrastructures):
        pretty = PrettyTable(["Resource name", "Status"])
        pretty.align = "l"

        print_table = False
        for infrastructure in infrastructures.items():
            print_table = True
            resource_name = infrastructure[0]
            status = "Pending"
            for resource_processed in infrastructures_resources_processed:
                key = list(resource_processed.keys())[0]
                if key == resource_name:
                    rp = resource_processed[key]
                    rc = rp['rc']
                    stderr = rp['stderr']
                    if rc != 0:
                        status = "Failed"
                    else:
                        if not (stderr == '' or stderr is None):
                            status = "OK, but stderr not empty"
                        else:
                            status = "OK"

            pretty.add_row([resource_name, status])

        if print_table:
            print(pretty)



import json
import urllib

import maccli.helper.http
from maccli.view.view_generic import show_error


def get_list():
    status_code, json, raw = maccli.helper.http.send_request("GET", "/resources")
    return json


def get(inf_name, inf_version, resource_name):
    params = {
        'inf_name': inf_name,
        'inf_version': inf_version,
        'name': resource_name
    }
    querystring = "?" + urllib.urlencode(params, True)
    status_code, json, raw = maccli.helper.http.send_request("GET", "/resource%s" % querystring)
    return json


def create(name, create_bash, rc, stderr, stdout, destroy_bash, metadata):
    """
     Creates a new resource
    """
    params = {
        'name': name,
        'create_cmd': create_bash,
        'create_rc': rc,
        'create_stderr': stderr,
        'create_stdout': stdout,
        'destroy_cmd': destroy_bash,
        'metadata': json.dumps(metadata),
    }

    json_request = json.dumps(params)

    status_code, json_response, raw = maccli.helper.http.send_request("POST", "/resource", data=json_request)

    if status_code == 400:
        show_error("Error while building request: " + raw)

    return json_response


def update(resource):

    json_request = json.dumps(resource)

    status_code, json_response, raw = maccli.helper.http.send_request("PUT", "/resource", data=json_request)

    if status_code == 400:
        show_error("Error while building request: " + raw)

    return json_response


def delete(resource):

    json_request = json.dumps(resource)

    status_code, json_response, raw = maccli.helper.http.send_request("DELETE", "/resource", data=json_request)

    if status_code == 400:
        show_error("Error while building request: " + raw)

    return json_response


import maccli
from prettytable import PrettyTable


def show_configurations(cookbooks):
    pretty = PrettyTable(["Tag", "Title", "Summary"])
    pretty.align = "l"
    if len(cookbooks):
        for cookbook in cookbooks:
            pretty.add_row([cookbook['tag'], cookbook['name'], cookbook['summary']])

        print(pretty)
        print("")
        print("Search more at %s/cookbooks" % maccli.domain)
        print("")
    else:
        print("")
        print("There are not configurations available")
        print("")


def show_configurations_url(cookbooks):
    pretty = PrettyTable(["Tag", "Url"])
    pretty.align = "l"
    if len(cookbooks):
        for cookbook in cookbooks:
            pretty.add_row([cookbook['tag'], cookbook['url']])

        print(pretty)
        print("")
        print("Search more at %s/cookbooks" % maccli.domain)
        print("")
    else:
        print("There are not configurations available")
        print("")
        print("You can create a new server configuration at %s" % maccli.domain)


def show_configurations_help():
    print("Show more help:")
    print("")
    print("    mac configuration -h")
    print("")






from prettytable import PrettyTable


def show_hardwares(hardwares):
    pretty = PrettyTable(["Id", "RAM", "Cores", "HD"])
    pretty.align = "r"
    pretty.align["Id"] = "l"
    for hardware in hardwares:
        processor_clean = ""
        for processor in hardware['processors']:
            if processor_clean != "":
                processor_clean += "\n"
            processor_clean += ("%s" % processor['cores'])

        volume_clean = ""
        for volume in hardware['volumes']:
            if volume_clean != "":
                volume_clean += "\n"

            volume_clean += ("%sGb" % volume['size'])

        pretty.add_row([hardware['id'], hardware['ram'], processor_clean, volume_clean])

    print(pretty)



import argparse


class AliasedSubParsersAction(argparse._SubParsersAction):
    old_init = staticmethod(argparse._ActionsContainer.__init__)

    @staticmethod
    def _containerInit(self, description, prefix_chars, argument_default, conflict_handler):
        AliasedSubParsersAction.old_init(self, description, prefix_chars, argument_default, conflict_handler)
        self.register('action', 'parsers', AliasedSubParsersAction)

    class _AliasedPseudoAction(argparse.Action):
        def __init__(self, name, aliases, help):
            dest = name
            if aliases:
                dest += ' (%s)' % ','.join(aliases)
            sup = super(AliasedSubParsersAction._AliasedPseudoAction, self)
            sup.__init__(option_strings=[], dest=dest, help=help)

    def add_parser(self, name, **kwargs):
        aliases = kwargs.pop('aliases', [])
        parser = super(AliasedSubParsersAction, self).add_parser(name, **kwargs)

        # Make the aliases work.
        for alias in aliases:
            self._name_parser_map[alias] = parser
        # Make the help text reflect them, first removing old help entry.
        if 'help' in kwargs:
            help = kwargs.pop('help')
            self._choices_actions.pop()
            pseudo_action = self._AliasedPseudoAction(name, aliases, help)
            self._choices_actions.append(pseudo_action)

        return parser


# override argparse to register new subparser action by default
argparse._ActionsContainer.__init__ = AliasedSubParsersAction._containerInit

# class AliasedSubParsersAction(argparse._SubParsersAction):
#     class _AliasedPseudoAction(argparse.Action):
#         def __init__(self, name, aliases, help):
#             dest = name
#             if aliases:
#                 dest += ' (%s)' % ','.join(aliases)
#             sup = super(AliasedSubParsersAction._AliasedPseudoAction, self)
#             sup.__init__(option_strings=[], dest=dest, help=help)
#
#     def add_parser(self, name, **kwargs):
#         if 'aliases' in kwargs:
#             aliases = kwargs['aliases']
#             del kwargs['aliases']
#         else:
#             aliases = []
#
#         parser = super(AliasedSubParsersAction, self).add_parser(name, **kwargs)
#
#         # Make the aliases work.
#         for alias in aliases:
#             self._name_parser_map[alias] = parser
#         # Make the help text reflect them, first removing old help entry.
#         if 'help' in kwargs:
#             help = kwargs.pop('help')
#             self._choices_actions.pop()
#             pseudo_action = self._AliasedPseudoAction(name, aliases, help)
#             self._choices_actions.append(pseudo_action)
#
#         return parser
#
#
# if __name__ == '__main__':
#     # An example parser with subcommands.
#
#     parser = argparse.ArgumentParser()
#     parser.register('action', 'parsers', AliasedSubParsersAction)
#     parser.add_argument("--library", metavar="libfile", type=str,
#                         help="library database filename")
#     subparsers = parser.add_subparsers(title="commands", metavar="COMMAND")
#
#     p_import = subparsers.add_parser("import", help="add files to library",
#                                      aliases=('imp', 'im'))
#     p_import.add_argument("filename", metavar="file", type=str, nargs="+",
#                           help="the files to import")
#
#     p_remove = subparsers.add_parser("remove", aliases=('rm',),
#                                      help="remove items")
#
#     args = parser.parse_args()
#     print(args)



import os
import subprocess
from rfc3987 import parse

from maccli.helper.exception import MacApiError, MacAuthError, BashException
import maccli


def update_pwd(path_or_url):
    try:
        parse(path_or_url, rule='URI')
    except ValueError as e:
        maccli.pwd = os.path.dirname(os.path.realpath(path_or_url))


def run(command):
    cmd_parts = ["/bin/bash", "-c", command]
    maccli.logger.debug("Running bash: %s " % command)
    try:
        process = subprocess.Popen(cmd_parts, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=maccli.pwd)
    except Exception as e:
        raise BashException("%s: %s" % (command, e))

    process.wait()
    rc = process.returncode
    stdout, stderr = process.communicate()
    maccli.logger.debug("STDOUT: %s" % stdout)
    maccli.logger.debug("STDERR: %s" % stderr)
    return rc, stdout.decode('utf-8', 'ignore'), stderr.decode('utf-8', 'ignore')



from urllib.parse import urljoin
from httplib2 import Http
import json
from urllib.parse import urlencode

from maccli.helper.exception import MacApiError, MacAuthError
import maccli


def get_auth_header(auth = None):
    if auth is None:
        if maccli.user and maccli.apikey:
            return {'Authorization': 'ApiKey %s:%s' % (maccli.user, maccli.apikey)}
        else:
            return {}
    else:
        return {'Authorization': 'Basic %s' % auth}


def send_request(method, path, auth=None, data=None, **kwargs):
    json_content = None
    url = urljoin(maccli.base_url, path.strip("/"))
    maccli.logger.info("%s %s %s %s" % (method, url, data, kwargs.get('data', '')))
    # construct headers
    headers = {'Content-Type': 'application/json', 'User-Agent': 'python-manageacloud/v%s' % maccli.__version__, 'Connection': 'close'}
    headers.update(get_auth_header(auth))
    # make the request
    h = Http()
    resp, content = h.request(url, method, headers=headers, body=data, **kwargs)
    # handle the response
    status_code = getattr(resp, 'status', None)
    maccli.logger.info("Status: %s", str(status_code))
    if not status_code:
        # Most likely network trouble
        raise MacApiError("No Response (%s %s)" % (method, url))
    elif 200 <= status_code <= 299:
        # Success
        if status_code != 204:
            # Try to parse the response.
            try:
                json_content = json.loads(content.decode('utf-8', 'ignore'))
            except Exception as e:
                maccli.logger.error("Error parsing json: %s", e)
                raise MacApiError("JSON Parse Error (%s %s). Response: %s" % (method, url, content))
        else:
            json_content = None
    else:
        # Server returned an error.
        if status_code == 403:
            raise MacAuthError("Not authorized")
        elif status_code == 503:
            maccli.logger.info("Status %s (%s %s). Response: %s" % (str(status_code), method, url, content))
        else:
            maccli.logger.warn("Status %s (%s %s). Response: %s" % (str(status_code), method, url, content))

    maccli.logger.info("Response: %s", json_content)
    return status_code, json_content, content


import getpass
import configparser
import logging
import os
import sys
import time
from os.path import join, expanduser
import traceback
import threading
import urllib
import urllib.request
import urllib.parse

# TODO implement progress bar
#from progressbar import ReverseBar, Percentage, ETA, RotatingMarker, Timer, ProgressBar

import maccli.service.auth
import maccli.service.instance
import maccli.service.provider
import maccli.service.macfile
import maccli.service.resource
import maccli.service.infrastructure
import maccli.facade.macfile
import maccli.service.configuration
import maccli.helper.macfile
import maccli.helper.cmd
import maccli.dao.inheritance
from maccli.view.view_generic import show_error, show
import maccli.view.view_location
import maccli.view.view_instance
import maccli.view.view_cookbook
import maccli.view.view_generic
import maccli.view.view_resource
import maccli.view.view_infrastructure
import maccli.view.view_hardware
from maccli.view.view_generic import GREEN, RED
from maccli.config import AUTH_SECTION, USER_OPTION, APIKEY_OPTION, MAC_FILE, EXCEPTION_EXIT_CODE, CONFIGURATION_FAILED, \
    CREATION_FAILED
from maccli.helper.exception import MacParseEnvException, MacErrorCreatingTier, MacParseParamException, \
    MacResourceException, MacJsonException


def help():
    maccli.view.view_generic.general_help()


def login():
    try:
        maccli.view.view_generic.show("")
        maccli.view.view_generic.show(
            "If you are using Manageacloud SaaS and you don't have credentials, please register at https://manageacloud.com")
        maccli.view.view_generic.show("")
        maccli.view.view_generic.show("If you are using Manageacloud Community please set the following variable:")
        maccli.view.view_generic.show("    export MAC=http://my_community_server/api/v1/")
        maccli.view.view_generic.show("")
        maccli.view.view_generic.show("More information at http://manageacloud/docs/getting-started/install")
        maccli.view.view_generic.show("")
        username = input("Username or email: ")
        password = getpass.getpass()

        user, api_key = maccli.service.auth.authenticate(username, password)
        if api_key is not None:
            config = configparser.ConfigParser()
            config.add_section(AUTH_SECTION)
            config.set(AUTH_SECTION, USER_OPTION, user)
            config.set(AUTH_SECTION, APIKEY_OPTION, api_key)
            with open(join(expanduser('~'), MAC_FILE), 'w') as cfgfile:
                config.write(cfgfile)
            print("Login succeeded!")

    except KeyboardInterrupt as e:
        show_error("")
        show_error("Authentication cancelled.")

    except Exception as e:
        show_error(e)
        sys.exit(EXCEPTION_EXIT_CODE)


def no_credentials():
    show("You need to login into Manageacloud.com")
    show()
    show("    mac login")
    show()
    show("If you do not have an account, register at https://manageacloud.com/register")


def instance_list():
    # try:
    json = maccli.service.instance.list_instances()
    maccli.view.view_instance.show_instances(json)

    # except Exception as e:
    #     show_error(e)
    #     sys.exit(EXCEPTION_EXIT_CODE)


def instance_ssh(raw_ids, command):
    # TODO move to parameter
    active_job_limit = 50

    if raw_ids == ["all"]:  # run in all instances
        ids = maccli.service.instance.list_instances()
    else:
        ids = maccli.service.instance.list_instances(name_or_ids=raw_ids)

    try:
        # array for job list
        jobs = {}

        maccli.logger.debug("Starting threads for command %s" % command)

        started_count = 0
        completed_count = 0
        total_count = len(ids)

        # bar = ProgressBar(maxval=total_count,widgets=[Percentage(), ReverseBar(), ETA(), RotatingMarker(), Timer()]).start()

        for id in ids:
            server_name = id["servername"]
            raw_id = id["id"]
            if command is None:
                maccli.service.instance.ssh_interactive_instance(id['id'])
            else:
                # define and add job
                t = threading.Thread(target=_run_cmd_simple, args=(server_name, raw_id, command))

                maccli.logger.debug("Starting job for %s. Active Jobs: %s." % (server_name, len(jobs)))
                if not t.is_alive():
                    started_count += 1
                    t.start()

                jobs[raw_id] = t

                # check if we are above the limit
                need_to_sleep = True
                while len(jobs) >= active_job_limit:
                    maccli.logger.debug("Job limit reached. Active Jobs: %s." % len(jobs))

                    keys = list(jobs.keys())
                    for key in keys:
                        if not jobs[key].is_alive():
                            completed_count += 1
                            # bar.update(completed_count)
                            show("Status: %s/%s/%s" % (completed_count, started_count, total_count))
                            jobs[key].join()
                            del jobs[key]
                            need_to_sleep = False
                        maccli.logger.debug("Job finished. Active Jobs: %s." % len(jobs))

                    if need_to_sleep:
                        time.sleep(1)

        # waiting for the existing thread to finish.
        # if a thread is too slow, we will report it
        slow_threads = {}
        while len(jobs) > 0:

            keys = list(jobs.keys())
            for key in keys:

                if not jobs[key].is_alive():
                    completed_count += 1
                    # bar.update(completed_count)
                    show("Status: %s/%s/%s" % (completed_count, started_count, total_count))
                    jobs[key].join()
                    del jobs[key]
                    if key in slow_threads:
                        del slow_threads[key]
                else:
                    if key in slow_threads:
                        if slow_threads[key] > 10:
                            show("Server %s is running the query slow: %s secs" % (key, slow_threads[key]))
                        slow_threads[key] += 1
                    else:
                        slow_threads[key] = 1

            time.sleep(1)

    except KeyboardInterrupt:
        show_error("Aborting")
    except Exception as e:
        tb = traceback.format_exc()
        show_error(e)
        show_error(tb)
        sys.exit(EXCEPTION_EXIT_CODE)


def _run_cmd_simple(server_name, raw_id, command):
    rc, stdout, stderr = maccli.service.instance.ssh_command_instance(raw_id, command)
    maccli.view.view_generic.showc("[%s] " % server_name, GREEN)
    if stdout:
        maccli.view.view_generic.show(stdout)
    elif stderr:
        maccli.view.view_generic.showc(raw_id, RED)
        maccli.view.view_generic.showc(stderr, RED)
    else:
        maccli.view.view_generic.show("\n")
    #maccli.view.view_generic.show()


def instance_create(cookbook_tag, bootstrap_raw, deployment, location, servername, provider, release_raw,
                    release_version_raw,
                    branch, hardware, lifespan, environments, hd, port, net):
    # allow format release:release_version, ie ubuntu:trusty
    if ":" in release_raw:
        release, release_version = release_raw.split(":", 1)
    else:
        release = release_raw
        release_version = release_version_raw

    # TODO check if cookbook_tag exists
    # TODO validate bootstrap inputs
    try:
        if cookbook_tag is None and bootstrap_raw is None:
            maccli.view.view_instance.show_instance_create_help()

        elif location is None:
            locations_json = maccli.service.provider.list_locations(provider, release)
            if locations_json is not None:
                show()
                show("--location parameter not set. You must choose the location.")
                show()
                show("Available locations:")
                show()
                if len(locations_json):
                    maccli.view.view_location.show_locations(locations_json)
                    maccli.view.view_instance.show_create_example_with_parameters(cookbook_tag, bootstrap_raw,
                                                                                  deployment,
                                                                                  locations_json[0]['id'], servername,
                                                                                  provider, release, release_version,
                                                                                  branch, hardware)

                else:
                    type = None
                    if release is not None:
                        type = release

                    if type is None and cookbook_tag is not None:
                        type = cookbook_tag

                    show("There is not locations available for '%s' and provider '%s'" % (type, provider))

                maccli.view.view_instance.show_instance_help()

        elif deployment == "production" and hardware is None or \
                deployment == "testing" and provider != "default" and hardware is None:
            hardwares = maccli.service.provider.list_hardwares(provider, location, release)
            show()
            show("--hardware not found. You must choose the hardware.")
            show()
            show("Available hardware:")
            show()
            maccli.view.view_hardware.show_hardwares(hardwares)
            if (len(hardwares) > 0):
                maccli.view.view_instance.show_create_example_with_parameters(cookbook_tag, bootstrap_raw, deployment,
                                                                              location, servername,
                                                                              provider, release, release_version,
                                                                              branch, hardwares[0]['id'])
        else:
            """ Execute create instance """

            # load the bootstrap file properly
            if bootstrap_raw is not None:
                if os.path.exists(bootstrap_raw):  # it is a file
                    # this is going to be the PWD to run commands
                    maccli.pwd = os.path.dirname(os.path.realpath(bootstrap_raw))
                    maccli.logger.info("Path %s exists, trying to open file", bootstrap_raw)
                    stream = open(bootstrap_raw, "r")
                    bootstrap = stream.read()
                elif _is_url(bootstrap_raw):  # try url
                    maccli.logger.info("%s looks like an URL, trying to open URL" % bootstrap_raw)
                    f = urllib.request.urlopen(bootstrap_raw)
                    bootstrap = f.read()
                else:  # values are bash executable
                    bootstrap = bootstrap_raw
            else:
                bootstrap = bootstrap_raw

            if cookbook_tag == "" and bootstrap == "":
                show_error("Server contains no configuration")
            else:
                instance = maccli.service.instance.create_instance(cookbook_tag, bootstrap, deployment, location,
                                                                   servername, provider, release, release_version,
                                                                   branch, hardware, lifespan, environments, hd, port,
                                                                   net)
                if instance is not None:
                    maccli.view.view_instance.show_instance(instance)

                maccli.view.view_generic.show("")
                maccli.view.view_generic.show("Monitor the progress of all servers:")
                maccli.view.view_generic.show("")
                maccli.view.view_generic.show("    watch mac instance list")
                maccli.view.view_generic.show("")
                maccli.view.view_generic.show("Tail server logs:")
                maccli.view.view_generic.show("")
                maccli.view.view_generic.show("    mac instance log -f %s" % instance['id'])
                maccli.view.view_generic.show("")
    except KeyboardInterrupt:
        show_error("Aborting")
    # except URLError:
    #     show_error("Can't open URL %s" % bootstrap_raw)
    except Exception as e:
        show_error(e)
        sys.exit(EXCEPTION_EXIT_CODE)


def _is_url(url):
    return bool(urllib.parse.urlparse(url).scheme)


def instance_destroy_help():
    maccli.view.view_instance.show_instance_destroy_help()


def instance_update_help():
    maccli.view.view_instance.show_instance_update_help()


def instance_ssh_help():
    maccli.view.view_instance.show_instance_ssh_help()


def instance_destroy(ids):
    try:
        maccli.logger.debug("Destroying instances %s " % ids)
        instances = []
        for instance_id in ids:
            maccli.logger.debug("Destroying instance %s " % instance_id)
            instance = maccli.service.instance.destroy_instance(instance_id)
            instances.append(instance)

        if instances is not None:
            maccli.view.view_instance.show_instances(instances)
    except KeyboardInterrupt:
        show_error("Aborting")
    except Exception as e:
        show_error(e)
        sys.exit(EXCEPTION_EXIT_CODE)


def instance_update(raw_ids, cookbook_tag, bootstrap):
    try:
        if raw_ids == ["all"]:  # run in all instances
            ids = maccli.service.instance.list_instances()
        else:
            ids = maccli.service.instance.list_instances(name_or_ids=raw_ids)

        instances = []
        for id in ids:
            maccli.logger.debug("Updating instance %s " % id['id'])
            instance = maccli.service.instance.update_configuration(cookbook_tag, bootstrap, id['id'])
            instances.append(instance)

        if instances is not None:
            maccli.view.view_instance.show_instances(instances)

        maccli.view.view_generic.show("")
        maccli.view.view_generic.show("To track configuration changes")
        maccli.view.view_generic.show("")
        for id in ids:
            maccli.view.view_generic.show("    mac instance log -f %s" % id['id'])

        maccli.view.view_generic.show("")

    except KeyboardInterrupt:
        show_error("Aborting")
    except Exception as e:
        show_error(e)
        sys.exit(EXCEPTION_EXIT_CODE)


def provider_help():
    maccli.view.view_generic.show_provider_help()


def credentials(provider, clientid, key_raw, force_file):
    """
    Save the credential's supplier if are correct.

    :param provider:
    :param clientid:
    :param key_raw: This might be a path. If so, the credentials are stored in the file.
    :return:
    """
    try:
        maccli.service.provider.save_credentials(provider, clientid, key_raw, force_file)
    except KeyboardInterrupt:
        show_error("Aborting")
    except Exception as e:
        show_error(e)
        sys.exit(EXCEPTION_EXIT_CODE)


def resouce_get_stdout(infrastructure_name, infrastructure_version, resource_name, key):
    try:
        output = maccli.service.resource.get_resource_value(infrastructure_name, infrastructure_version, resource_name,
                                                            key)
        if isinstance(output, str):
            sys.stdout.write(output)
        else:
            print(output)

    except KeyError:
        show_error("Key '%s' not found" % key)
    except KeyboardInterrupt:
        show_error("Aborting")
    except Exception as e:
        show_error(e)
        sys.exit(EXCEPTION_EXIT_CODE)


def instance_help():
    maccli.view.view_instance.show_instance_help()


def configuration_list():
    try:
        configurations = maccli.service.configuration.list_configurations()
        maccli.view.view_cookbook.show_configurations(configurations)
    except KeyboardInterrupt:
        show_error("Aborting")
    except Exception as e:
        show_error(e)
        sys.exit(EXCEPTION_EXIT_CODE)


def configuration_search(keywords, show_url):
    try:
        configurations = maccli.service.configuration.search_configurations(keywords)
        if show_url:
            maccli.view.view_cookbook.show_configurations_url(configurations)
        else:
            maccli.view.view_cookbook.show_configurations(configurations)
    except KeyboardInterrupt:
        show_error("Aborting")
    except Exception as e:
        show_error(e)
        sys.exit(EXCEPTION_EXIT_CODE)


def configuration_help():
    maccli.view.view_cookbook.show_configurations_help()


def convert_to_yaml(args):
    yaml = maccli.service.macfile.convert_args_to_yaml(args)
    maccli.view.view_generic.show(yaml)


def process_macfile(file, resume, params, quiet, on_failure):
    try:
        # save the PWD
        maccli.helper.cmd.update_pwd(file)

        raw = maccli.helper.macfile.load_macfile(file)

        macfile_raw = None
        try:
            macfile_raw = maccli.dao.inheritance.resolve_inheritance(raw, params)
        except MacParseParamException as e:
            maccli.view.view_generic.show_error(e)
            exit(11)

        root, roles, infrastructures, actions, resources = (None,) * 5
        try:
            root, roles, infrastructures, actions, resources = maccli.service.macfile.parse_macfile(macfile_raw)
        except MacParseParamException as e:
            maccli.view.view_generic.show_error(e)
            exit(12)

        if not resume:

            existing_infrastructures = maccli.service.infrastructure.search_instances(root['name'], root['version'])

            if len(existing_infrastructures) > 0:
                maccli.view.view_generic.show()
                maccli.view.view_generic.show()
                maccli.view.view_generic.show_error(
                    "There are active instances for infrastructure '%s' and version '%s'" % (
                    root['name'], root['version']))
                maccli.view.view_generic.show()
                maccli.view.view_generic.show()
                maccli.view.view_infrastructure.show_infrastructure_instances(existing_infrastructures)
                maccli.view.view_infrastructure.show_resources_in_infrastructure(existing_infrastructures)
                maccli.view.view_generic.show()
                maccli.view.view_generic.show()
                maccli.view.view_generic.show(
                    "Instances and resources must be destroyed before attempting to create another "
                    "infrastructure using the same version.")
                maccli.view.view_generic.show("")
                maccli.view.view_generic.show("To destroy the complete infrastructure:")
                maccli.view.view_generic.show(
                    "    mac infrastructure destroy <infrastructure name> <infrastructure version>")
                maccli.view.view_generic.show("")
                maccli.view.view_generic.show("To view the infrastructure available:")
                maccli.view.view_generic.show("    mac infrastructure list")
                maccli.view.view_generic.show("")
                maccli.view.view_generic.show("To view the resources and instances in a infrastructure:")
                maccli.view.view_generic.show("    mac infrastructure items")
                maccli.view.view_generic.show("")

                exit(7)

            if quiet:
                maccli.view.view_generic.show("Infrastructure %s version %s" % (root['name'], root['version']))
            else:
                maccli.view.view_generic.header("Infrastructure %s version %s" % (root['name'], root['version']), "=")

        infrastructure_error_detail = None
        finish = False
        infrastructure_resources_failed = False
        infrastructure_resources_processed = []  # looks for the non-instance infrastructure
        processing_instances = maccli.service.instance.list_by_infrastructure(root['name'], root['version'])
        while not finish:
            if not quiet:
                maccli.view.view_generic.clear()
                maccli.view.view_instance.show_instances(processing_instances)
                maccli.view.view_infrastructure.show_infrastructure_resources(infrastructures,
                                                                              infrastructure_resources_processed)

            # apply configuration to the instances
            maccli.facade.macfile.apply_instance_infrastructure_changes(processing_instances, root['name'],
                                                                        root['version'], quiet, infrastructures,
                                                                        infrastructure_resources_processed)

            # process resources
            finish = True
            try:
                processed_resources_part, finish_resources = maccli.facade.macfile.apply_resources(processing_instances,
                                                                                                   infrastructure_resources_processed,
                                                                                                   processing_instances,
                                                                                                   roles,
                                                                                                   infrastructures,
                                                                                                   actions, resources,
                                                                                                   root, quiet)
                maccli.logger.debug("Resources processed this run: %s " % processed_resources_part)
                infrastructure_resources_processed = infrastructure_resources_processed + processed_resources_part
                maccli.logger.debug("Total resources processed: %s " % infrastructure_resources_processed)
                finish = finish and finish_resources
            except MacResourceException as e:
                infrastructure_error_detail = [e.__str__()]
                infrastructure_resources_failed = True
                finish = True

            # check instances has been processed
            processing_instances = maccli.service.instance.list_by_infrastructure(root['name'], root['version'])
            for instance in processing_instances:
                if not (instance['status'].startswith("Ready") or instance['status'] == CREATION_FAILED or instance[
                    'status'] == CONFIGURATION_FAILED):
                    finish = False
                if on_failure is not None and (
                        instance['status'] == CREATION_FAILED or instance['status'] == CONFIGURATION_FAILED):
                    finish = finish and True

            if infrastructure_resources_failed:
                finish = True
            else:
                if not finish:
                    time.sleep(3)

        instances_processed = maccli.service.instance.list_by_infrastructure(root['name'], root['version'])
        if not quiet:
            maccli.view.view_generic.clear()
        maccli.view.view_instance.show_instances(instances_processed)
        maccli.view.view_infrastructure.show_infrastructure_resources(infrastructures,
                                                                      infrastructure_resources_processed)
        maccli.view.view_generic.show("")
        maccli.view.view_resource.show_resources(infrastructure_resources_processed)

        # Clean up if failure
        instances_failed = maccli.facade.macfile.clean_up(instances_processed, on_failure)

        if instances_failed or infrastructure_resources_failed:
            maccli.view.view_generic.show("")
            maccli.view.view_generic.showc("Infrastructure failed.", RED)

            # Of the error is in infrastructure, display it.
            if infrastructure_error_detail is not None:
                maccli.view.view_generic.show("")
                if 'stderr' in infrastructure_error_detail:
                    maccli.view.view_generic.show(infrastructure_error_detail['stderr'])
                else:
                    maccli.view.view_generic.show(infrastructure_error_detail)

                if 'cmd' in infrastructure_error_detail:
                    maccli.view.view_generic.show(infrastructure_error_detail['cmd'])

            # if an instance failed, the details of the error are in the log
            maccli.view.view_generic.show("")
            maccli.view.view_generic.show("Logs available at")
            maccli.view.view_generic.show("    mac instance log <instance id or name>")
            exit(12)
        else:
            maccli.view.view_generic.showc("Infrastructure created successfully.", GREEN)
            maccli.view.view_generic.show("")
    except KeyboardInterrupt:
        show_error("Aborting")
    except Exception as e:
        if logging.getLogger().getEffectiveLevel() == logging.DEBUG:
            traceback.print_exc(file=sys.stdout)
        else:
            show_error(e)

        sys.exit(EXCEPTION_EXIT_CODE)


def instance_fact(instance_id):
    try:
        json = maccli.service.instance.facts(instance_id)
        maccli.view.view_instance.show_facts(json)
    except KeyboardInterrupt:
        show_error("Aborting")
    except Exception as e:
        show_error(e)
        sys.exit(EXCEPTION_EXIT_CODE)


def instance_log(instance_id, follow):
    try:
        if follow:
            while True:
                logs = maccli.service.instance.log(instance_id, follow)
                if logs == "":
                    time.sleep(5)
                else:
                    maccli.view.view_generic.show(logs)
                    time.sleep(1)
        else:
            json = maccli.service.instance.log(instance_id, False)
            maccli.view.view_instance.show_logs(json)

    except KeyboardInterrupt:
        show_error("Aborting")
    except Exception as e:
        show_error(e)
        sys.exit(EXCEPTION_EXIT_CODE)


def instance_lifespan(instance_id, amount):
    try:
        instance = maccli.service.instance.lifespan(instance_id, amount)
        maccli.view.view_instance.show_instance(instance)
    except KeyboardInterrupt:
        show_error("Aborting")
    except Exception as e:
        show_error(e)
        sys.exit(EXCEPTION_EXIT_CODE)


def infrastructure_list():
    try:
        infrastructure = maccli.service.infrastructure.list_infrastructure()
        maccli.view.view_infrastructure.show_infrastructure(infrastructure)
    except KeyboardInterrupt:
        show_error("Aborting")
    except Exception as e:
        show_error(e)
        sys.exit(EXCEPTION_EXIT_CODE)


def infrastructure_search(name, version):
    try:
        infrastructure = maccli.service.infrastructure.search_instances(name, version)
        if len(infrastructure):
            maccli.view.view_infrastructure.show_infrastructure_instances(infrastructure)
            maccli.view.view_generic.show()
            maccli.view.view_infrastructure.show_resources_in_infrastructure(infrastructure)
        else:
            maccli.view.view_generic.show("There are not active infrastructure")
    except KeyboardInterrupt:
        show_error("Aborting")
    except Exception as e:
        show_error(e)
        sys.exit(EXCEPTION_EXIT_CODE)


def infrastructure_update(name, version, cookbook_tag):
    try:
        infrastructures = maccli.service.infrastructure.search_instances(name, version)

        if len(infrastructures):
            infrastructure = infrastructures[0]

            instances = []
            for instance in infrastructure['cloudServers']:
                instance_updated = maccli.service.instance.update_configuration(cookbook_tag, "", instance['id'])
                instances.append(instance_updated)

            if instances is not None:
                maccli.view.view_instance.show_instances(instances)
        else:
            maccli.view.view_generic.show("Infrastructure '%s' version '%s' not found" % (name, version))

    except KeyboardInterrupt:
        show_error("Aborting")
    except Exception as e:
        if logging.getLogger().getEffectiveLevel() == logging.DEBUG:
            traceback.print_exc(file=sys.stdout)
        else:
            show_error(e)
        sys.exit(EXCEPTION_EXIT_CODE)


def infrastructure_destroy(name, version):
    try:
        infrastructures = maccli.service.infrastructure.search_instances(name, version)

        if len(infrastructures):
            infrastructure = infrastructures[0]

            for instance in infrastructure['cloudServers']:
                maccli.view.view_generic.showc("Instance %s marked as deleted" % instance['id'], GREEN)
                maccli.view.view_generic.show("")
                maccli.service.instance.destroy_instance(instance['id'])

            if len(infrastructure['cloudServers']):
                time.sleep(5)  # give some time to instances to free resources

            for resource in infrastructure['resources']:
                try:
                    maccli.facade.macfile.destroy_resource(resource, infrastructure['cloudServers'],
                                                           infrastructure['resources'])
                except MacJsonException as e:
                    maccli.view.view_generic.showc("\nError while destroying resource %s\n\n" % resource['name'], RED)
                    maccli.view.view_generic.showc("Error while navigating json! %s\n" % e.message, RED)

        else:
            maccli.view.view_generic.show("Infrastructure '%s' version '%s' not found" % (name, version))

    except KeyboardInterrupt:
        show_error("Aborting")
    except Exception as e:
        if logging.getLogger().getEffectiveLevel() == logging.DEBUG:
            traceback.print_exc(file=sys.stdout)
        else:
            show_error(e)
        sys.exit(EXCEPTION_EXIT_CODE)


def infrastructure_lifespan(amount, name, version):
    try:
        infrastructure = maccli.service.infrastructure.lifespan(amount, name, version)
        maccli.view.view_infrastructure.show_infrastructure_instances(infrastructure)
    except KeyboardInterrupt:
        show_error("Aborting")
    except Exception as e:
        show_error(e)
        sys.exit(EXCEPTION_EXIT_CODE)


def infrastructure_ssh_keys(name, version, known_host):
    """
    Diplays ssh keys or adds it to known_hosts

    :param name:
    :param version:
    :param known_host:
    :return:
    """
    try:
        ssh_keys = maccli.service.infrastructure.keys(name, version, known_host)
        if not maccli.quiet:
            if known_host:
                for ssh_key in ssh_keys:
                    show("%s processed" % ssh_key['cloudServer']['ipv4'])

            else:
                for ssh_key in ssh_keys:
                    show(ssh_key['stdout'])

    except KeyboardInterrupt:
        show_error("Aborting")
    except Exception as e:
        show_error(e)
        sys.exit(EXCEPTION_EXIT_CODE)



from collections import OrderedDict
import os
import urllib
from functools import reduce

import yaml
import yaml.representer
import re

from maccli.view.view_generic import show_error
from maccli.helper.exception import MacParamValidationError, MacParseParamException
from maccli.helper.unsortable import ordered_load
import maccli
from maccli.helper.unsortable import UnsortableOrderedDict
import maccli.dao.inheritance


import textwrap


""" Yaml file format
description: Manageacloud CLI
version: 0.1a6
timestamp: '2015-02-05 11:59:07'
roles:
  default:
    instance create:
      branch: master
      configuration: basic_ubuntu_1404
      deployment: testing
      environment:
      - DBNAME: pgbench
      - PGUSER: benchuser
      hardware: https://www.googleapis.com/compute/v1/projects/soy-sound-613/zones/asia-east1-a/machineTypes/f1-micro
      location: asia-east1-a
      name: localtesting
      provider: gce
      release: any
"""


def convert_args_to_yaml(args):
    maccli.logger.debug("Converting options %s: " % args)
    key = ""
    if args.cmd is not None and args.subcmd is not None:
        key = args.cmd + ' ' + args.subcmd
    elif args.cmd is not None:
        key = args.cmd

    attributes = filter(
        lambda a: not a.startswith('_') and a not in ["cmd", "subcmd", "yaml", "debug", "verbose", "quiet"], dir(args))

    # defines the parameter that must exist, even if empty
    empty_parameters = ['name']

    params_instance = UnsortableOrderedDict()
    params_infrastructure = UnsortableOrderedDict()
    for attr in attributes:
        if attr in empty_parameters:
            value = ""
        else:
            value = getattr(args, attr)

        if value is not None:
            if attr in ['hardware', 'location', 'provider', 'deployment', 'lifespan', 'name', 'release']:
                params_infrastructure[attr] = value
            else:
                params_instance[attr] = value

    # default roles, what tier can be created
    params_role = UnsortableOrderedDict()
    params_role[key] = params_instance
    role_name = "default"
    roles = UnsortableOrderedDict()
    roles["default"] = params_role

    # default infrastructures, create one
    params_infrastructure['role'] = role_name
    params_infrastructure['amount'] = 1
    infrastructures = UnsortableOrderedDict()
    infrastructures["default"] = params_infrastructure

    # general structure
    data = UnsortableOrderedDict()
    data["mac"] = maccli.__version__
    data["description"] = "Manageacloud CLI"
    data["name"] = "manageacloud.com"
    data["version"] = "1.0"
    data["roles"] = roles
    data["infrastructures"] = infrastructures

    yaml.add_representer(UnsortableOrderedDict, yaml.representer.SafeRepresenter.represent_dict)
    return yaml.dump(data, default_flow_style=False, sort_keys=False)


def validate_param(actual, expected, optional=None):
    unexpected = is_unexpected(actual, expected)
    if optional is not None:
        unexpected_optional = is_unexpected(unexpected, optional)
    else:
        unexpected_optional = unexpected

    if _ilen(unexpected_optional):
        error = "Incorrect file format. The following parameters are unexpected:\n"
        for p in unexpected_optional:
            error += " - %s\n" % p
        error += "\n"
        error += "Required: %s\n" % expected
        if optional is not None:
            error += "Optional: %s\n" % optional
        raise MacParamValidationError(error)

    notpresent = is_present(actual, expected)
    if optional is not None:
        notpresent_optional = is_unexpected(notpresent, optional)
    else:
        notpresent_optional = notpresent

    if _ilen(notpresent_optional):
        error = "Incorrect file format. The following parameters are needed and not present:\n"
        for p in notpresent_optional:
            error += " - %s\n" % p
        raise MacParamValidationError(error)


def is_present(actual, expected):
    """ evaluates if all params in actual exist in expected  """
    if expected is None:
        notfound = actual
    else:
        notfound = filter(lambda x: x not in actual, expected)
    return notfound


def is_unexpected(actual, expected):
    """ evaluates if there is a parameter in actual that does not exist in expected  """
    if expected is None:
        unexpected = actual
    else:
        unexpected = filter(lambda x: x not in expected, actual)
    return unexpected


def parse_macfile(raw):
    """

    Parse and validates the macfile and returns structured contents

    :param raw: R
    :return: root, role and infrastructure contents

    TODO remove "exit" from this method and raise exceptions
    """
    # validate root
    root_params = ['mac', 'version', 'name', 'description', 'infrastructures']
    root_params_optional = ['actions', 'resources', 'roles']
    raw_root_keys = raw.keys()
    try:
        validate_param(raw_root_keys, root_params, root_params_optional)
    except MacParamValidationError as e:
        show_error(e)
        exit(1)

    # validate roles
    expected_roles = []
    role_root_params = ["instance create"]
    role_params = []  # mandatory parameters
    role_optional_params = ['branch', 'hd', 'lifespan', 'environment', 'configuration', 'bootstrap bash']

    if 'roles' in raw:
        if raw['roles'] is None:
            show_error("Roles section defined but empty")
            exit(1)
        else:
            raw_role_root_keys = raw['roles'].keys()
            for key_role_root in raw_role_root_keys:
                expected_roles.append(key_role_root)
                raw_role_keys = raw['roles'][key_role_root].keys()
                try:
                    validate_param(raw_role_keys, role_root_params)
                    for key_role in raw_role_keys:
                        raw_role = raw['roles'][key_role_root][key_role].keys()
                        validate_param(raw_role, role_params, role_optional_params)
                except MacParamValidationError as e:
                    show_error(e)
                    exit(1)

                if 'configuration' in raw['roles'][key_role_root]['instance create'] and 'bootstrap bash' in raw['roles'][key_role_root]['instance create']:
                    raise MacParseParamException("'configuration' or 'bootstrap bash' are exclusive at role '%s'" % key_role_root)
                elif not ('configuration' in raw['roles'][key_role_root]['instance create'] or 'bootstrap bash' in raw['roles'][key_role_root]['instance create']):
                    raise MacParseParamException("'configuration' or 'bootstrap bash' is required for role '%s'" % key_role_root)

                if 'environment' in raw['roles'][key_role_root]['instance create']:
                    environment = raw['roles'][key_role_root]['instance create']['environment']
                    if not isinstance(environment, list):
                        error_text = textwrap.dedent(
                                    """
                                    'environment' should be a list, however '%s' found.

                                    HINT: the format for 'environment' in role '%s' should be:
                                    environment:
                                     - KEY1=VAR1
                                     - KEY2=VAR2
                                    """ % (type(environment), key_role_root))
                        raise MacParseParamException(error_text)

    else:
        raw['roles'] = []

    # validate infrastructures
    raw_infrastructure_root_keys = raw['infrastructures'].keys()
    actual_roles = []
    for key_infrastructure_root in raw_infrastructure_root_keys:
        raw_infrastructure_keys = raw['infrastructures'][key_infrastructure_root].keys()
        if 'role' in raw_infrastructure_keys:
            """ Infrastructure related with a role """
            infrastructure_optional_params = ['lifespan', 'deployment', 'release', 'provider', 'hardware', 'amount',
                                              'environment', 'ready', 'net']
            infrastructure_root_params = ['amount', 'role', 'hardware', 'location', 'provider', 'name', 'deployment',
                                          'release']
            infrastructure_root_params_mac = ['amount', 'role', 'location', 'provider', 'name', 'deployment',
                                              'release']

            try:
                provider = raw['infrastructures'][key_infrastructure_root]['provider']
            except:
                provider = ""

            try:
                if provider == "manageacloud":
                    validate_param(raw_infrastructure_keys, infrastructure_root_params_mac, infrastructure_optional_params)
                else:
                    validate_param(raw_infrastructure_keys, infrastructure_root_params, infrastructure_optional_params)
            except MacParamValidationError as e:
                show_error(e)
                exit(1)

            actual_roles.append(raw['infrastructures'][key_infrastructure_root]["role"])
        else:
            """ Infrastructure related with a resource"""
            infrastructure_optional_params = ['resource', 'action', 'ready', 'params']
            validate_param(raw_infrastructure_keys, None, infrastructure_optional_params)

        # check that the format of 'ready' is role.app
        if 'ready' in raw['infrastructures'][key_infrastructure_root]:
            ready_value = raw['infrastructures'][key_infrastructure_root]['ready']
            ready_regex = re.compile('role\.[a-zA-Z\+]')
            #print(ready_value)
            if not ready_regex.match(ready_value):
                show_error("'ready' parameter on '%s' at infrastructure section should have the format 'role.my_role_name'" % key_infrastructure_root)
                exit(1)

        # check if there is at least a 'resource' 'action' or 'role'
        if not ('resource' in raw['infrastructures'][key_infrastructure_root] or 'action' in raw['infrastructures'][key_infrastructure_root] or
                'role' in raw['infrastructures'][key_infrastructure_root]):
            show_error("Infrastructure '%s' requires at least one value" % key_infrastructure_root)
            exit(1)

    # validate actions
    if 'actions' in raw.keys():
        actions_optional_params = ["ssh", "bash"]
        raw_actions_root_keys = raw['actions'].keys()
        for key_action_root in raw_actions_root_keys:
            if isinstance(raw['actions'][key_action_root], OrderedDict):
                raw_action_keys = raw['actions'][key_action_root].keys()
                if len(raw_action_keys) != 1:
                    print("The action '%s' does not have anything defined." % key_action_root)
                    exit(1)
                validate_param(raw_action_keys, None, actions_optional_params)
            else:
                print("The action '%s' has wrong format." % key_action_root)
                exit(1)

    else:
        raw['actions'] = []

    # validate resources
    if 'resources' in raw.keys():
        resources_optional_params = ["create bash", "destroy bash"]
        raw_resources_root_keys = raw['resources'].keys()
        for key_resource_root in raw_resources_root_keys:
            if isinstance(raw['resources'][key_resource_root], OrderedDict):
                raw_resource_keys = raw['resources'][key_resource_root].keys()
                if len(raw_resource_keys) < 1:
                    print("The resource '%s' does not have anything defined." % key_resource_root)
                    exit(1)
                validate_param(raw_resource_keys, None, resources_optional_params)
            else:
                print("The resource '%s' has wrong format." % key_resource_root)
                exit(1)

    else:
        raw['resources'] = []

    # check the values of infrastructures > default > role
    not_existing_roles = is_unexpected(actual_roles, expected_roles)
    if _ilen(not_existing_roles):
        print("The following roles are used under 'infrastructures' but are never defined:")
        for p in not_existing_roles:
            print(" - %s" % p)
        exit(3)

    # check the values of infrastructures > default > role
    not_existing_roles = is_present(actual_roles, expected_roles)
    if _ilen(not_existing_roles):
        print("WARNING! The following roles are defined but never user:")
        for p in not_existing_roles:
            print(" - %s" % p)

    # get the root parameters
    root = {
        'version': raw['version'],
        'name': raw['name'],
    }

    return root, raw['roles'], raw['infrastructures'], raw['actions'], raw['resources']


def _ilen(iterable):
    return reduce(lambda sum, element: sum + 1, iterable, 0)


import unittest

import mock

import maccli.helper.metadata
from tests.mock_data import *


class HelperMetadataTestCase(unittest.TestCase):

    def test_metadata(self):
        EXPECTED_META = {'macfile_infrastructure_name': 'consul01', 'macfile_role_name': 'consul', 'version': '1.0', 'environment_raw': [OrderedDict([('MY_IP', 'consul01.PRIVATE_IP')]), OrderedDict([('MEMBERS_IP', 'consul02.PRIVATE_IP')])], 'name': 'consul'}
        macfile_root = {'version': '1.0', 'name': 'consul'}
        infrastructure_key = "consul01"
        role_key = "consul"
        role = OrderedDict([('branch', 'master'), ('configuration', 'consul')])
        infrastructure = OrderedDict([('deployment', 'testing'), ('location', 'us-central1-c'), ('name', 'consul01'), ('role', 'consul'), ('environment', [OrderedDict([('MY_IP', 'consul01.PRIVATE_IP')]), OrderedDict([('MEMBERS_IP', 'consul02.PRIVATE_IP')])])])
        actual_meta = maccli.helper.metadata.metadata_instance(macfile_root, infrastructure_key, role_key, role, infrastructure)
        self.assertEqual(actual_meta, EXPECTED_META)

    def test_metadata_version_string(self):
        EXPECTED_META = {'macfile_infrastructure_name': 'consul01', 'macfile_role_name': 'consul', 'version': '1', 'environment_raw': [OrderedDict([('MY_IP', 'consul01.PRIVATE_IP')]), OrderedDict([('MEMBERS_IP', 'consul02.PRIVATE_IP')])], 'name': 'consul'}
        macfile_root = {'version': 1, 'name': 'consul'}
        infrastructure_key = "consul01"
        role_key = "consul"
        role = OrderedDict([('branch', 'master'), ('configuration', 'consul')])
        infrastructure = OrderedDict([('deployment', 'testing'), ('location', 'us-central1-c'), ('name', 'consul01'), ('role', 'consul'), ('environment', [OrderedDict([('MY_IP', 'consul01.PRIVATE_IP')]), OrderedDict([('MEMBERS_IP', 'consul02.PRIVATE_IP')])])])
        actual_meta = maccli.helper.metadata.metadata_instance(macfile_root, infrastructure_key, role_key, role, infrastructure)
        self.assertEqual(actual_meta, EXPECTED_META)


    def test_get_environment(self):
        ROLE = OrderedDict([('branch', 'master'), ('configuration', 'consul'), ('environment', [OrderedDict([('MEMBERS_IP', 'consul.PRIVATE_IP')])])])
        INFRASTRUCTURE = OrderedDict([('deployment', 'testing'), ('location', 'us-central1-c'), ('name', 'consul01'), ('role', 'consul'), ('environment', [OrderedDict([('MY_IP', 'consul01.PRIVATE_IP')])])])
        EXPECTED_ENVIRONMENT = [OrderedDict([('MEMBERS_IP', 'consul.PRIVATE_IP')]), OrderedDict([('MY_IP', 'consul01.PRIVATE_IP')])]
        actual_environment = maccli.helper.metadata.get_environment(ROLE, INFRASTRUCTURE)
        self.assertEqual(actual_environment, EXPECTED_ENVIRONMENT)

    def test_metadata_resource(self):
        EXPECTED_META = {'macfile_infrastructure_name': 'consul01', 'version': '1.0', 'name': 'consul', 'macfile_resource_name': 'resource_name_test'}
        macfile_root = {'version': '1.0', 'name': 'consul'}
        infrastructure_key = "consul01"
        resource_name = "resource_name_test"
        actual_meta = maccli.helper.metadata.metadata_resource(macfile_root, infrastructure_key, resource_name, [])
        self.assertEqual(actual_meta, EXPECTED_META)

    def test_metadata_resource_version_string(self):
        EXPECTED_META = {'macfile_infrastructure_name': 'consul01', 'version': '1', 'name': 'consul', 'macfile_resource_name': 'resource_name_test'}
        macfile_root = {'version': 1, 'name': 'consul'}
        infrastructure_key = "consul01"
        resource_name = "resource_name_test"
        actual_meta = maccli.helper.metadata.metadata_resource(macfile_root, infrastructure_key, resource_name, [])
        self.assertEqual(actual_meta, EXPECTED_META)

    def test_metadata_resource_params(self):
        EXPECTED_META = {'macfile_infrastructure_name': 'consul01', 'version': '1', 'name': 'consul', 'macfile_resource_name': 'resource_name_test', 'macfile_infrastructure_params': [OrderedDict([('param1', 'value1')])]}
        macfile_root = {'version': 1, 'name': 'consul'}
        infrastructure_key = "consul01"
        resource_name = "resource_name_test"
        infrastructure = OrderedDict([('params', [OrderedDict([('param1', 'value1')])])])
        actual_meta = maccli.helper.metadata.metadata_resource(macfile_root, infrastructure_key, resource_name, infrastructure )
        self.assertEqual(actual_meta, EXPECTED_META)




import re
import os
import codecs

from setuptools import setup, find_packages


def read(*parts):
    path = os.path.join(os.path.dirname(__file__), *parts)
    with codecs.open(path, encoding='utf-8') as fobj:
        return fobj.read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')


def find_long_description():
    this_directory = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(this_directory, 'README.rst')) as f:
        long_description = f.read()
    return long_description


with open('requirements.txt') as f:
    install_requires = f.read().splitlines()

setup(
    name='mac',
    long_description=find_long_description(),
    long_description_content_type='text/x-rst',
    version=find_version('maccli', '__init__.py'),
    packages=find_packages(),
    install_requires=install_requires,
    entry_points={
        'console_scripts':
            ['mac = maccli.mac_cli:main']
    },
    include_package_data=True,
    author='Manageacloud Services Pty LTD',
    author_email='support@manageacloud.com',
    description='Technology agnostic orchestration framework',
    license='Apache v2',
    keywords='manageacloud environment infrastructure deployment blue green docker puppet chef saltstack ansible bash rackspace aws digitalocean azure'
             'gce google compute engine multicloud multi-cloud orchestration ci cd continuous integration delivery deployment a/b testing',
    url='https://manageacloud.com',
    test_suite='tests',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: POSIX',
        'Programming Language :: Python'

    ]

)



import unittest

import mock

import maccli.service.infrastructure
from tests.mock_data import *

MOCK_INFRASTRUCTURE = [{u'cloudServers': [{u'status': u'Ready', u'servername': u'mct-lb-eh4', u'lifespan': 67, u'ipv4': u'130.211.132.25', u'type': u'testing', u'id': u'eh4gpld06l3itv18k3hvbnrumu', u'metadata': {u'infrastructure': {u'macfile_infrastructure_name': u'lb', u'environment_raw': [{u'APP_IP': u'app.PRIVATE_IP'}], u'version': u'2.0', u'name': u'demo', u'macfile_role_name': u'lb'}, u'system': {u'infrastructure': {u'hardware': u'https://www.googleapis.com/compute/v1/projects/manageacloud-instances/zones/REPLACE_ZONE/machineTypes/f1-micro', u'deployment': u'testing', u'location': u'us-central1-c', u'lifespan': 90, u'provider': u'manageacloud'}, u'role': {u'environment': {u'APP_IP': [u'10.240.229.100']}, u'cookbook_tag': u'loadbalancer', u'block_tags': [{}]}}}}], u'name': u'demo', u'versions': [u'2.0']}]
MOCK_INFRASTRUCTURE_300 = [{u'cloudServers': [{u'status': u'Ready', u'servername': u'mct-lb-eh4', u'lifespan': 300, u'ipv4': u'130.211.132.25', u'type': u'testing', u'id': u'eh4gpld06l3itv18k3hvbnrumu', u'metadata': {u'infrastructure': {u'macfile_infrastructure_name': u'lb', u'environment_raw': [{u'APP_IP': u'app.PRIVATE_IP'}], u'version': u'2.0', u'name': u'demo', u'macfile_role_name': u'lb'}, u'system': {u'infrastructure': {u'hardware': u'https://www.googleapis.com/compute/v1/projects/manageacloud-instances/zones/REPLACE_ZONE/machineTypes/f1-micro', u'deployment': u'testing', u'location': u'us-central1-c', u'lifespan': 300, u'provider': u'manageacloud'}, u'role': {u'environment': {u'APP_IP': [u'10.240.229.100']}, u'cookbook_tag': u'loadbalancer', u'block_tags': [{}]}}}}], u'name': u'demo', u'versions': [u'2.0']}]
MOCK_INSTANCE_300 = {u'status': u'Ready', u'servername': u'mct-lb-eh4', u'lifespan': 300, u'ipv4': u'130.211.132.25', u'type': u'testing', u'id': u'eh4gpld06l3itv18k3hvbnrumu', u'metadata': {u'infrastructure': {u'macfile_infrastructure_name': u'lb', u'environment_raw': [{u'APP_IP': u'app.PRIVATE_IP'}], u'version': u'2.0', u'name': u'demo', u'macfile_role_name': u'lb'}, u'system': {u'infrastructure': {u'hardware': u'https://www.googleapis.com/compute/v1/projects/manageacloud-instances/zones/REPLACE_ZONE/machineTypes/f1-micro', u'deployment': u'testing', u'location': u'us-central1-c', u'lifespan': 300, u'provider': u'manageacloud'}, u'role': {u'environment': {u'APP_IP': [u'10.240.229.100']}, u'cookbook_tag': u'loadbalancer', u'block_tags': [{}]}}}}
NAME = "demo"


class ServiceInfrastructureTestCase(unittest.TestCase):

    @mock.patch('maccli.dao.api_infrastructure.search_infrastructure')
    @mock.patch('maccli.dao.api_instance.update')
    def test_lifespan_name(self, mock_update, mock_search):
        mock_search.return_value = 200, MOCK_INFRASTRUCTURE
        mock_update.return_value = MOCK_INSTANCE_300
        json_response = maccli.service.infrastructure.lifespan(300, NAME, None)
        self.assertTrue(mock_update.called)
        self.assertEqual(json_response, MOCK_INFRASTRUCTURE_300)




import unittest

import mock

import maccli.view.view_resource
from tests.mock_data import *

RESOURCES_PROCESSED = [{'build_lb_inf': {'rc': 0, 'stderr': '', 'stdout': '{\n    "DNSName": "my-load-balancer-1110685180.us-east-1.elb.amazonaws.com"\n}\n'}}, {'register_lb_inf': {'rc': 0, 'stderr': '', 'stdout': '{\n    "Instances": [\n        {\n            "InstanceId": "i-d3911600"\n        }\n    ]\n}\n'}}]

class ResourcesViewTestCase(unittest.TestCase):

    def test_show_resources(self):
        maccli.view.view_resource.show_resources(RESOURCES_PROCESSED)



import os
import unittest

from tests.mock_data import *
import maccli.helper.cmd

#
#   For this unit testing, I needed to evaluate how popen behaves,
#   therefore subprocess.Popen is not mocked up. Ideas are welcomed!
#


class CmdTestCase(unittest.TestCase):

    def setUp(self):
        maccli.pwd = os.getcwd()

    def test_working_command(self):
        rc, stdout, stderr = maccli.helper.cmd.run("true")
        self.assertEqual(rc, 0)
        self.assertEqual(stdout, "")
        self.assertEqual(stderr, "")

    def test_not_found(self):
        rc, stdout, stderr = maccli.helper.cmd.run("command_not_found")
        self.assertEqual(rc, 127)
        self.assertEqual(stdout, "")
        self.assertEqual(stderr, '/bin/bash: command_not_found: command not found\n')

    def test_stdout(self):
        rc, stdout, stderr = maccli.helper.cmd.run("echo hello")
        self.assertEqual(rc, 0)
        self.assertEqual(stdout, "hello\n")
        self.assertEqual(stderr, "")

    def test_stderr(self):
        rc, stdout, stderr = maccli.helper.cmd.run("date invalid_date")
        self.assertEqual(rc, 1)
        self.assertEqual(stdout, "")
        self.assertEqual(stderr, 'date: invalid date ‘invalid_date’\n')

    def test_environment_variables(self):
        rc, stdout, stderr = maccli.helper.cmd.run('MY_ENV=1 env | grep MY_ENV')
        self.assertEqual(rc, 0)
        self.assertEqual(stdout, 'MY_ENV=1\n')
        self.assertEqual(stderr, "")

    def test_single_quotes(self):
        rc, stdout, stderr = maccli.helper.cmd.run("ls '*'")
        self.assertEqual(rc, 2)
        self.assertEqual(stdout, '')
        self.assertEqual(stderr, 'ls: cannot access \'*\': No such file or directory\n')

    def test_double_quotes(self):
        rc, stdout, stderr = maccli.helper.cmd.run('ls "*"')
        self.assertEqual(rc, 2)
        self.assertEqual(stdout, '')
        self.assertEqual(stderr, 'ls: cannot access \'*\': No such file or directory\n')

    def test_absolute_path(self):
        path = '/this/is/absolute'
        myfile = path + '/path'
        maccli.helper.cmd.update_pwd(myfile)
        self.assertEqual(maccli.pwd, path)

    def test_relative_path(self):
        path = 'is/relative'
        myfile = path + '/path'
        maccli.helper.cmd.update_pwd(myfile)
        self.assertEqual(maccli.pwd, os.getcwd() + "/" + path)

    def test_url(self):
        old_pwd = maccli.pwd
        maccli.helper.cmd.update_pwd('https://manageacloud.com/my/path')
        self.assertEqual(maccli.pwd, old_pwd)



import json
import os
import urllib
from maccli.helper.exception import InstanceNotReadyException, InstanceDoesNotExistException, BashException, \
    MacParameterNotFound, MacJsonException, MacParseParamException
import maccli.service.instance
__author__ = 'tk421'
import re
import maccli
import maccli.helper.cmd
import maccli.dao.api_instance


def load_macfile(path):
    """
    :param path: initial file load
    :return:
    """
    if os.path.exists(path) or os.path.exists("%s/%s" % (maccli.pwd, path)):  # open file
        # this is going to be the PWD to run commands
        if os.path.exists("%s/%s" % (maccli.pwd, path)):
            path = "%s/%s" % (maccli.pwd, path)

        maccli.logger.info("Path %s exists, trying to open file", path)
        stream = open(path, "r")
        contents = stream.read()
    else:  # try url
        maccli.logger.info("Path does not exists, assuming %s is an URL" % path)
        f = urllib.urlopen(path)
        contents = f.read()

    return contents


def has_dependencies(text, roles, infrastructures, actions):
    """ check of there are dependencies in the text
        a dependency is somthing with the format:
         - role.[role_name].[action]
         - infrastructure.[infrastructure name].[action]

        returns true of false

    """
    # TODO add validation for resources
    # TODO add validation for cases where name is incorrect. e.g infrastructure.image_base.get_id instead infrastructure.image_base_inf.get_id
    has_deps = False
    matches = get_dependencies(text)
    if matches:
        for match in matches:
            type_name = match[0]
            name = match[1]
            action = match[2]
            maccli.logger.debug("1 - Match found: type '%s' name '%s' action '%s' " % (type_name, name, action))
            if type_name == "role" and name in roles or type_name == "infrastructure" and name in infrastructures:
                if action in actions:
                    has_deps = True
                elif name in infrastructures and action in infrastructures[name]:
                    has_deps = True
                else:
                    maccli.logger.warn("%s.%s.%s has been found but %s does not match with an action"
                                       % (type_name, name, action, action))

            elif type_name == "infrastructure" and name == "param":
                # TODO check if the param actually exists for every infrastructure that calls that resource
                has_deps = True

            elif type_name == "resource":  # TODO add more validation
                has_deps = True

            elif type_name == "action":
                has_deps = True

            else:
                maccli.logger.warn("'%s.%s.%s' has been found but we do not how to process '%s' "
                                   % (type_name, name, action, type_name))
    maccli.logger.debug("has_dependencies? %s" % has_deps)
    return has_deps


def is_role_dependencies_ready(infrastructure, processed_instances, infrastructure_key):
    """ Return if the role dependencies are ready """
    ready = True # we just return a boolean we are not ready
    if 'ready' in infrastructure:  # check if infrastructure has dependencies
        instances_ready = infrastructure['ready']  # role.app
        maccli.logger.debug("Infrastructure %s infrastructure_key requires %s ready before proceeding" % (infrastructure_key, instances_ready))
        instance_type, role_name = instances_ready.split(".")

        for instance in processed_instances:
            instance_role_name = instance['metadata']['infrastructure']['macfile_role_name']
            instance_infrastructure_name = instance['metadata']['infrastructure']['macfile_infrastructure_name']
            if instance_role_name == role_name or instance_infrastructure_name == role_name:
                if not ("Error" in instance['status'] or instance['status'].startswith("Ready")):
                    maccli.logger.info("%s is not ready yet, waiting ...", instance['id'])
                    ready = False
                    break  # exit from loop to avoid processing other resources
    return ready


def get_dependencies(text):
    """ check of there are dependencies in the text
        a dependency is somthing with the format:
         - role.[role_name].[action]
         - infrastructure.[infrastructure name].[action]
         - resource.something.text.regex(myregex)
        returns true of false
    """
    try:
        a = re.compile("(role|infrastructure|resource|action)\.([a-zA-Z0-9_\-\.]*?)\.([a-zA-Z0-9_\-\.]*|text\.regex\((.+)\))($|\s|\"|')", re.IGNORECASE)
        matches = a.findall(text)
        maccli.logger.debug("Searching for dependencies at %s" % text)
    except TypeError:  # not an string, no dependencies
        maccli.logger.debug("'%s' is not an string", text)
        matches = []

    return matches


def get_action_ssh(name, actions):
    toreturn = ""
    for action_key in actions:
        if action_key == name:
            toreturn = actions[action_key]['ssh']
            break
    return toreturn


def get_action_bash(name, actions):
    toreturn = ""
    for action_key in actions:
        if action_key == name:
            toreturn = actions[action_key]['bash']
            break
    return toreturn


def dict_has_dependencies(dict, roles, infrastructures, actions):
    """
    Check if dictionary's values has dependencies

    :param dict:
    :param roles:
    :param infrastructures:
    :param actions:
    :return:
    """
    toreturn = False
    for key in dict.keys():
        text_raw = dict[key]
        if has_dependencies(text_raw, roles, infrastructures, actions):
            toreturn = True
            break
    return toreturn


def parse_envs_dict(dict, instances, roles, infrastructures, actions, processed_resources):
    """
    Loops over a dictionary to find values that must be processed.

    :param dict: dictionary input
    :param instances:
    :param roles:
    :param infrastructures:
    :param actions:
    :param processed_resources:
    :return: dictionary with values processed, and if all values could be processed
    """
    total_processed = True
    for key in dict.keys():
        text_raw = dict[key]
        if isinstance(text_raw, str):
            text, processed = parse_envs(text_raw, instances, roles, infrastructures, actions, processed_resources)
            total_processed = total_processed and processed
            if processed:
                dict[key] = text

    return dict, total_processed


def parse_envs(text, instances, roles, infrastructures, actions, processed_resources, infrastructure=None):
    # print("=============")
    # print(text)
    # print(instances)
    # print(roles)
    # print(infrastructures)
    # print(actions)
    # print(processed_resources)
    # print(infrastructure)
    # print("=============")

    """
        replace the dependencies

    :param text: text that contains dependencies
    :param instances: instances from the context
    :param roles: roles available in the context
    :param infrastructures: all the infrastructures available in the context
    :param actions: all the actions available in the context
    :param processed_resources: all the processed resources, including output of commands
    :param infrastructure: the infrastructure that we are executing. Not available for all the contexts.
    :return:
    """

    all_processed = True

    # let's get all the dependencies of variables that we have to substitute
    matches = get_dependencies(text)

    # loop every variable
    if matches:
        for match in matches:
            type_name = match[0]
            name = match[1]
            action = match[2]
            maccli.logger.debug("2 - Match found: type '%s' name '%s' action '%s' " % (type_name, name, action))

            # now we process the variables depending on the strategy to solve

            # We check if the matches are processed. If it is not possible to process in this iteration, we might
            # not have all the required information yet.
            match_processed = False

            # parse values from "infrastructures" section in the macfile
            if name in infrastructures and action in infrastructures[name]:
                #  search in infrastructures
                #  the substitution is an infrastructure
                value = infrastructures[name][action]
                text = text.replace("%s.%s.%s" % (type_name, name, action), value)
                match_processed = True

            # parse values from "infrastructures.params" section in the macfile. This is a bit different because keys are
            # arbitrary
            elif type_name == "infrastructure" and name == "param":

                if not infrastructure:
                    maccli.logger.warn("Infrastructure context not provided to substitute %s.%s.%s" % (type_name, name, action))

                elif 'params' not in infrastructure:
                    maccli.logger.warn("Infrastructure context does not have params %s" % infrastructure)

                elif action in infrastructure['params']:

                    # the params value could require a substitution
                    value_raw = infrastructure['params'][action]
                    if has_dependencies(value_raw, roles, infrastructures, actions):
                        value, processed = parse_envs(value_raw, instances, roles, infrastructures, actions, processed_resources, infrastructure)
                    else:
                        processed = True
                        value = value_raw

                    if processed:
                        text = text.replace("%s.%s.%s" % (type_name, name, action), str(value))
                        match_processed = True

                else:
                    raise MacParameterNotFound("Can't find value for parameter %s.%s.%s" % (type_name, name, action))


            # match values that are processed resources
            elif any(name in d for d in processed_resources) and not (action in actions):
                # search in resources
                for processed_resource in processed_resources:
                    if name in processed_resource:
                        value_raw = processed_resource[name]['stdout']

                        try:
                            text_format, text_id_raw = action.split(".", 1)
                        except ValueError:
                            raise MacParameterNotFound("The value %s in the parameter %s.%s does not have the proper format." % (action, type_name, name))

                        # output format is json
                        if text_format == "json":
                            try:
                                texts = text_id_raw.split(".")
                                value_json = json.loads(value_raw.strip())
                                value = value_json
                                for text_part in texts:
                                    if text_part.isdigit():
                                        value = value[int(text_part)]
                                    else:
                                        value = value[text_part]

                                text = text.replace("%s.%s.%s" % (type_name, name, action), value)
                                match_processed = True
                            except KeyError:
                                if not(value_raw == '' or value_raw is None):
                                    original_task = "%s.%s.%s" % (type_name, name, action)
                                    maccli.logger.warn("We cannot find '%s' at '%s'" % (original_task, name))
                                    maccli.logger.debug("Original value: %s" % value_raw)
                                match_processed = False

                        # output format is text, and it will be processed with a regular expression
                        elif text_format == "text":
                            try:
                                regex_pattern = match[3]
                                maccli.logger.debug("Regex marching: %s at value %s " % (regex_pattern, value_raw))
                                regex = re.compile(regex_pattern, re.IGNORECASE)
                                matches = regex.findall(value_raw.strip())
                                maccli.logger.debug("Matches %s" % matches)
                                value = matches[0]
                                text = text.replace("%s.%s.%s" % (type_name, name, action), value)
                                match_processed = True
                            except Exception as e:
                                maccli.logger.debug("Error matching regex %s at value %s because of %s" % (match[3], value_raw, e))
                                maccli.logger.warn("Error matching regex %s at value %s" % (match[3], value_raw))
                                match_processed = False

                        else:
                            raise NotImplementedError

            # process if it is an action, but it is bash and will be executed from
            # the machine that is running the macfile
            elif type_name == "action" and name in actions:
                # Executes the action, it is currently adhoc only for 'json'

                bash_command_raw = get_action_bash(name, actions)

                # bash command might have parameters to be replaced
                if has_dependencies(bash_command_raw, roles, infrastructures, actions):
                    bash_command, processed = parse_envs(bash_command_raw, instances, roles, infrastructures, actions, processed_resources, infrastructure)
                else:
                    bash_command = bash_command_raw

                rc = None
                stdout = None
                try:
                    rc, stdout, stderr = maccli.helper.cmd.run(bash_command)
                except Exception as e:
                    stderr = e.message
                    maccli.logger.warn("Error executing %s: %s" % (bash_command, e))

                if rc is not None:
                    if rc == 0:
                        parts = action.split(".")
                        action_type = parts.pop(0)
                        value_raw = stdout.strip()

                        # original value, just in case we do not find a match
                        value = "%s.%s.%s" % (type_name, name, action)

                        # get value from json structure
                        if action_type == "json":
                            value_json = json.loads(value_raw)
                            recursive_value = value_json
                            for part in parts:
                                if part.isdigit():
                                    recursive_value = recursive_value[int(part)]
                                else:
                                    recursive_value = recursive_value[part]

                            value = recursive_value

                        # output format is text, and it will be processed with a regular expression
                        elif action_type == "text":
                            try:
                                regex_pattern = match[3]
                                maccli.logger.debug("Regex marching: %s at value %s " % (regex_pattern, stdout))
                                regex = re.compile(regex_pattern, re.IGNORECASE)
                                matches = regex.findall(stdout.strip())
                                maccli.logger.debug("Matches %s" % matches)
                                value = matches[0]
                            except Exception as e:
                                maccli.logger.debug("Error matching regex %s at value %s because of %s" % (match[3], stdout, e))
                                maccli.logger.warn("Error matching regex %s at value %s" % (match[3], stdout))

                        else:
                            raise NotImplementedError

                        text = text.replace("%s.%s.%s" % (type_name, name, action), value)
                        match_processed = True
                    else:
                        maccli.logger.warn("Error executing bash action %s: %s" % (bash_command, stderr))
                        #raise BashException("Error executing bash action %s: %s" % (bash_command, stderr), stderr)
                        match_processed = False

            # action is related with an instance, and it will be executed via SSH
            elif action in actions:
                # search in actions
                outputs = []

                # execute the action in an instance
                for instance in instances:
                    if type_name == "role" and name in roles:
                        matching_name = instance['metadata']['infrastructure']['macfile_role_name']
                    elif type_name == "infrastructure" and name in infrastructures:
                        matching_name = instance['metadata']['infrastructure']['macfile_infrastructure_name']
                    else:
                        #  this method ignores the replacement
                        matching_name = ""

                    if matching_name and matching_name == name:
                        ssh_command = get_action_ssh(action, actions)
                        rc = None
                        stderr = None
                        try:
                            rc, ssh_raw, stderr = maccli.service.instance.ssh_command_instance(instance['id'], ssh_command)
                        except InstanceNotReadyException:
                            rc = -1
                        finally:
                            if rc is not None:
                                if rc == 0:
                                    outputs.append(ssh_raw)
                                    match_processed = True
                                else:
                                    if stderr is not None:
                                        maccli.logger.warn("Error executing ssh action %s: %s" % (ssh_command, stderr))

                                        # auto accept ssh keys if the servers was just created
                                        if "Host key verification failed" in stderr:
                                            maccli.dao.api_instance.sshkeys(instance['ipv4'], True)

                                    match_processed = False
                                    break

                text = text.replace("%s.%s.%s" % (type_name, name, action), " ".join(outputs))
            else:
                # This parameter cannot be executed
                if not name in infrastructures:
                    raise MacParameterNotFound("The parameter %s.%s has not been found while processing %s " % (type_name, name, text))
                else:
                    maccli.logger.warn("The parameter %s.%s.%s has not been found" % (type_name, name, action))

            all_processed = all_processed and match_processed

            if not all_processed:
                break

    return text, all_processed


def parse_envs_destroy(resource_to_destroy, instances, resources):
    """ parse envs for the destroy event
            - resource.[infrastructure_name].json.jsonValue
            - infrastructure.param.paramname
    """
    # print ("================")
    # print (resource_to_destroy)
    # print (instances)
    # print (resources)
    # print ("================")
    text = resource_to_destroy['cmdDestroy']
    matches = get_dependencies(text)
    if matches:
        for match in matches:
            type_name = match[0]
            name = match[1]
            action = match[2]
            maccli.logger.debug("3 - Match found: type '%s' name '%s' action '%s' " % (type_name, name, action))

            if type_name == "resource":
                for resource in resources:
                    if name == resource['metadata']['infrastructure']['macfile_infrastructure_name']:
                        parts = action.split(".")
                        action_type = parts.pop(0)
                        value_raw = resource['create']['stdout'].strip()

                        # original value, just in case we do not find a match
                        value = "%s.%s.%s" % (type_name, name, action)

                        # get value from json structure
                        if action_type == "json":
                            value_json = json.loads(value_raw)
                            recursive_value = value_json
                            for part in parts:
                                if part.isdigit():
                                    recursive_value = recursive_value[int(part)]
                                else:
                                    try:
                                        recursive_value = recursive_value[part]
                                    except TypeError:
                                        raise MacJsonException("The parameter %s doesn't fit at %s.\n\n\n"
                                                               "Are you trying to operate a list as a dictionary ?\n\n" % (part, recursive_value))

                            value = recursive_value

                        elif action_type == "text":
                            try:
                                regex_pattern = match[3]
                                maccli.logger.debug("Regex marching: %s at value %s " % (regex_pattern, value_raw))
                                regex = re.compile(regex_pattern, re.IGNORECASE)
                                matches = regex.findall(value_raw.strip())
                                maccli.logger.debug("Matches %s" % matches)
                                value = matches[0]
                            except Exception as e:
                                maccli.logger.debug("Error matching regex %s at value %s because of %s" % (match[3], value_raw, e))
                                maccli.logger.warn("Error matching regex %s at value %s" % (match[3], value_raw))
                        else:
                            raise NotImplementedError

                        text = text.replace("%s.%s.%s" % (type_name, name, action), value)

            elif type_name == "infrastructure" and name == "param":

                if 'macfile_infrastructure_params' in resource_to_destroy['metadata']['infrastructure']:
                    if action in resource_to_destroy['metadata']['infrastructure']['macfile_infrastructure_params']:
                        value = resource_to_destroy['metadata']['infrastructure']['macfile_infrastructure_params'][action]
                        text = text.replace("%s.%s.%s" % (type_name, name, action), value)
                    else:
                        maccli.logger.warn("Param %s not found when processing %s.%s.%s to destroy resource" % (action, type_name, name, action))
                else:
                    maccli.logger.warn("Warning while destroying resource! Params are not available and are required for %s.%s.%s" % (type_name, name, action))
    return text


def parse_params(raw, params_raw):
    """
    The content of macfiles might have parameters to parse.
    """

    clean = raw
    params_found = []
    if params_raw is not None:
        for param in params_raw:
            key, value = param.split("=", 1)
            clean_tmp = clean.replace("{%s}" % key, value)
            if clean != clean_tmp:
                params_found.append(key)
            clean = clean_tmp

    if re.search(r'{([a-zA-Z_\-]*?)}', clean, re.MULTILINE):
        raise MacParseParamException("MACFILE contains parameters that were not available:\n"
                                     "\n"
                                     "%s"
                                     "\n"
                                     "Available parameters %s" % (clean, str(params_raw)))

    return clean, params_found


import json
from maccli.helper.exception import FactError

import maccli.helper.http
import maccli.helper.cmd
from maccli.view.view_generic import show_error


def get_list():
    status_code, json, raw = maccli.helper.http.send_request("GET", "/instances")

    return json


def credentials(instance_id):

    status_code, json_response, raw = maccli.helper.http.send_request("GET", "/credential/%s" % instance_id)

    if status_code == 404:
        show_error("Server not found")

    if status_code == 400:
        show_error("There is a problem with the input parameters")

    return json_response


def create(cookbook_tag, bootstrap, deployment, location, servername, provider, release, release_version,
           branch, hardware, lifespan, environments, hd, port, net, metadata, applyChanges):
    """
     Creates a new instance

    :param cookbook_tag:
    :param bootstrap:
    :param deployment:
    :param location:
    :param servername:
    :param provider:
    :param release:
    :param release_version:
    :param branch:
    :param hardware:
    :param lifespan:
    :param environments:
    :param hd:
    :return:
    """

    params = {
        'cookbook_tag': cookbook_tag,
        'bootstrap': bootstrap,
        'deployment': deployment,
        'location': location,
        'servername': servername,
        'provider': provider,
        'release': release,
        'release_version': release_version,
        'branch': branch,
        'hardware': hardware,
        'lifespan': lifespan,
        'environments': environments,
        'hd': hd,
        'port': port,
        'net': net,
        'metadata': json.dumps(metadata),
        'apply_changes': applyChanges
    }

    json_request = json.dumps(params)

    status_code, json_response, raw = maccli.helper.http.send_request("POST", "/instance", data=json_request)

    if status_code == 400:
        show_error("Error while building request: " + raw)

    if status_code == 409:
        show_error("Resources limit reached. If you need more resources please contact "
                   "sales@manageacloud.com. Detailed output:" + raw)

    return json_response


def update_configuration(cookbook_tag, bootstrap, instance_id, new_metadata):
    """
     Creates a new instance

    :param cookbook_tag:
    :param instance_id:
    :return:
    """

    params = {
        'cookbook_tag': cookbook_tag,
        'bootstrap': bootstrap,
        'instance_id': instance_id,
        'metadata': new_metadata
    }

    json_request = json.dumps(params)

    status_code, json_response, raw = maccli.helper.http.send_request("PUT", "/instance", data=json_request)

    if status_code == 400:
        show_error("Error while building request: " + raw)

    if status_code == 404:
        show_error("Error while building request: " + raw)

    if status_code == 401:
        show_error("Error while building request: " + raw)

    return json_response


def destroy(instanceid):

    status_code, json_response, raw = maccli.helper.http.send_request("DELETE", "/instance/%s" % instanceid)

    if status_code == 404:
        show_error("Server %s not found" % instanceid)

    if status_code == 400:
        show_error("Error with parameters: %s " % raw)

    return json_response


def facts(instance_id):

    status_code, json_response, raw = maccli.helper.http.send_request("GET", "/facts/%s" % instance_id)

    if status_code == 404:
        show_error("Server not found")

    if status_code == 400:
        show_error("There is a problem with the input parameters")

    if status_code == 503:
        raise FactError("Facts are not available yet.")

    return json_response


def log(instance_id):

    status_code, json_response, raw = maccli.helper.http.send_request("GET", "/log/%s" % instance_id)

    if status_code == 404:
        show_error("Server not found")

    if status_code == 400:
        show_error("There is a problem with the input parameters")

    return json_response


def log_follow(instance_id):

    status_code, json_response, raw = maccli.helper.http.send_request("GET", "/logf/%s" % instance_id)

    if status_code == 404:
        show_error("Server not found")

    if status_code == 400:
        show_error("There is a problem with the input parameters")

    return json_response['payload']


def update(instance_id, lifespan):

    params = {
        'lifespan': lifespan
    }

    json_request = json.dumps(params)

    status_code, json_response, raw = maccli.helper.http.send_request("PUT", "/instance/%s" % instance_id,
                                                                      data=json_request)

    if status_code == 404:
        show_error("Server not found")

    if status_code == 400:
        show_error("There is a problem with the input parameters")

    return json_response


def sshkeys(ip, known_host):
    """

    Enable the ssh fingerprint in the system

    :param ip: ip to check
    :param known_host: if the fingerprint should be added to the known_host file
    :return:
    """
    # scans and displays the ssh public key
    CMD_KEYSCAN = "ssh-keyscan %s"

    # add keys to known_host if the key is unique
    CMD_KNOWNHOST = "mkdir -p ~/.ssh && touch ~/.ssh/known_hosts && " + CMD_KEYSCAN + " 2>&1 | sort -u - ~/.ssh/known_hosts > ~/.ssh/tmp_hosts && mv ~/.ssh/tmp_hosts ~/.ssh/known_hosts"

    if known_host:
        cmd = CMD_KNOWNHOST % ip
    else:
        cmd = CMD_KEYSCAN % ip

    rc, stdout, stderr = maccli.helper.cmd.run(cmd)

    return rc, stdout, stderr



import maccli.dao.api_configuration


def list_configurations():
    """
        List available configurations for the current user
    """

    server_status, response = maccli.dao.api_configuration.get_user_configuration()

    return response


def search_configurations(keywords):
    """
        List available configurations for the current user
    """

    server_status, response = maccli.dao.api_configuration.search_public_configuration(keywords)

    return response





import unittest

import mock

import maccli.service.provider
from maccli.config import RELEASE_ANY
from tests.mock_data import *


DEFAULT_CONFIGURATION = "cookbook_tag"
DEFAULT_PROVIDER = "manageacloud"
DEFAULT_LOCATION = "sfo1"
DEFAULT_RELEASE = RELEASE_ANY
RELEASE_AMAZON = "amazon"


class AuthTestCase(unittest.TestCase):
    @mock.patch('maccli.dao.api_provider.get_locations')
    def testlist_locations(self, mock):
        mock.return_value = (200, MOCK_LOCATION_LIST_JSON)
        json_response = maccli.service.provider.list_locations(DEFAULT_PROVIDER, DEFAULT_RELEASE)
        self.assertTrue(mock.called)
        self.assertEqual(json_response, MOCK_LOCATION_LIST_JSON)

    @mock.patch('maccli.dao.api_provider.get_locations')
    def testlist_locations(self, mock):
        mock.return_value = (200, MOCK_LOCATION_LIST_RELEASES_JSON)
        json_response = maccli.service.provider.list_locations(DEFAULT_PROVIDER, RELEASE_AMAZON)
        self.assertTrue(mock.called)
        self.assertEqual(json_response, MOCK_LOCATION_LIST_RELEASES_AMAZON_JSON)


    @mock.patch('maccli.dao.api_provider.get_hardwares')
    def test_list_hardwares(self, mock):
        mock.return_value = (200, MOCK_HARDWARE_LIST_JSON)
        json_response = maccli.service.provider.list_hardwares(DEFAULT_PROVIDER, DEFAULT_LOCATION, DEFAULT_RELEASE)
        self.assertTrue(mock.called)
        self.assertEqual(json_response, MOCK_HARDWARE_LIST_JSON)



import sys

from prettytable import PrettyTable
import maccli.view.view_generic


def show_instances(instances):
    pretty = PrettyTable(["Instance name", "IP", "Instance ID", "Type", "Status"])

    if len(instances):
        for instance in instances:
            type = instance['type']
            if 'lifespan' in instance and instance['lifespan'] > 0 and instance['status'] == "Ready":
                status = "%s (%im left)" % (instance['status'], instance['lifespan'])
            else:
                status = instance['status']

            try:
                name = instance['metadata']['infrastructure']['name']
                version = instance['metadata']['infrastructure']['version']
                if name and version:
                    type = "%s %s" % (name, version)
            except KeyError:
                # not dangerous, infrastructure information is not available
                maccli.logger.debug("Key error while getting infrastructure information")

            pretty.add_row([instance['servername'], instance['ipv4'], instance['id'], type, status])
        print(pretty)
    else:
        print("There are no active instances")


def show_instance(instance):
    pretty = PrettyTable(["Instance name", "IP", "Instance ID", "Type", "Status"])
    if 'lifespan' in instance and instance['lifespan'] > 0 and instance['status'] == "Ready":
        status = "%s (%im left)" % (instance['status'], instance['lifespan'])
    else:
        status = instance['status']

    pretty.add_row([instance['servername'], instance['ipv4'], instance['id'], instance['type'], status])
    print(pretty)


def show_instance_create_locations_example(cookbook_tag, locationid):
    print("")
    print("Example:")
    print("")
    print("    mac instance create -c %s -l %s" % (cookbook_tag, locationid))
    print("")


def show_instance_create_help():
    print("--configuration parameter is required. ")
    print("")
    print("You can list your configurations:")
    print("")
    print("    mac configuration list")
    print("")
    print("or search public configurations")
    print("")
    print("    mac configuration search")
    print("")
    print("To create a new instance with this configuration:")
    print("")
    print("    mac instance create -c <configuration tag>")
    show_instance_help()


def show_instance_help():
    print("")
    print("Show more help:")
    print("")
    print("    mac instance -h")
    print("")


def show_instance_destroy_help():
    print("")
    print("Show more help:")
    print("")
    print("    mac instance destroy -h")
    print("")


def show_instance_update_help():
    print("")
    print("Show more help:")
    print("")
    print("    mac instance update -h")
    print("")

def show_instance_ssh_help():
    print("")
    print("Show more help:")
    print("")
    print("    mac ssh -h")
    print("")


def show_create_example_with_parameters(cookbook_tag, bootstrap, deployment, location, servername, provider,
                                        release, release_version, branch, hardware):
    output = "mac instance create "

    if cookbook_tag is not None and cookbook_tag != "":
        output += " -c " + cookbook_tag

    if bootstrap is not None and bootstrap != "":
        output += " -b " + bootstrap

    if deployment is not None and deployment != "testing":
        output += " -d " + deployment

    if servername is not None and servername != "":
        output += " -n " + servername

    if provider is not None and provider != "default":
        output += " -p " + provider

    if release is not None and release != "any":
        if release_version:
            output += " -r %s:%s" % (release, release_version)
        else:
            output += " -r %s" % release

    if branch is not None and branch != "master":
        output += " -r " + branch

    if location is not None and location != "":
        output += " -l " + location

    if hardware is not None and hardware != "":
        output += " -hw " + hardware

    print("")
    print("Example:")
    print("")
    print("    %s" % output)
    print("")


def show_facts(facts):
    for key, value in facts.iteritems():
        print("%s: %s" % (key, value))


def show_logs(logs):
    print("")
    print("")
    if len(logs['cloudServerLogs']) > 0:
        for log in logs['cloudServerLogs']:
            maccli.view.view_generic.header("Creation logs for %s" % log['created'])
            print(log['text'])
    else:
        print("There is not creation logs")

    print("")
    print("")
    if len(logs['cloudServerBlockLogs']) > 0:
        for log in logs['cloudServerBlockLogs']:
            maccli.view.view_generic.header("Configuration logs for %s" % log['created'])
            print(log['text'])
    else:
        print("There is not configuration logs")


def show_processing_instances(instances):
    instances_statuses = {}
    for instance in instances:

        if instance['status'] not in instances_statuses:
            instances_statuses[instance['status']] = 1
        else:
            instances_statuses[instance['status']] = instances_statuses[instance['status']] + 1

    sys.stdout.write("\033[K")
    sys.stdout.write("Progress: ")
    for key in instances_statuses:
        sys.stdout.write("%s: %s " % (key, instances_statuses[key]))
    sys.stdout.write(" \r")
    sys.stdout.flush()


import re


def is_ip_private(ip):
    # https://en.wikipedia.org/wiki/Private_network

    priv_lo = re.compile("^127\.\d{1,3}\.\d{1,3}\.\d{1,3}$")
    priv_24 = re.compile("^10\.\d{1,3}\.\d{1,3}\.\d{1,3}$")
    priv_20 = re.compile("^192\.168\.\d{1,3}.\d{1,3}$")
    priv_16 = re.compile("^172.(1[6-9]|2[0-9]|3[0-1]).[0-9]{1,3}.[0-9]{1,3}$")

    return priv_lo.match(ip) or priv_24.match(ip) or priv_20.match(ip) or priv_16.match(ip)


def is_local(ip):
    # https://en.wikipedia.org/wiki/Private_network

    priv_lo = re.compile("^127\.\d{1,3}\.\d{1,3}\.\d{1,3}$")

    return priv_lo.match(ip)